# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: google/protobuf/any.proto, google/protobuf/api.proto, google/protobuf/descriptor.proto, google/protobuf/duration.proto, google/protobuf/empty.proto, google/protobuf/field_mask.proto, google/protobuf/source_context.proto, google/protobuf/struct.proto, google/protobuf/timestamp.proto, google/protobuf/type.proto, google/protobuf/wrappers.proto
# plugin: python-aristaproto

import warnings
from typing import TYPE_CHECKING


if TYPE_CHECKING:
    from dataclasses import dataclass
else:
    from pydantic.dataclasses import dataclass

from typing import (
    Dict,
    List,
    Mapping,
    Optional,
)

from pydantic import root_validator
from typing_extensions import Self

import aristaproto
from aristaproto.utils import hybridmethod


class Syntax(aristaproto.Enum):
    """The syntax in which a protocol buffer element is defined."""

    PROTO2 = 0
    """Syntax `proto2`."""

    PROTO3 = 1
    """Syntax `proto3`."""

    EDITIONS = 2
    """Syntax `editions`."""


class FieldKind(aristaproto.Enum):
    """Basic field types."""

    TYPE_UNKNOWN = 0
    """Field type unknown."""

    TYPE_DOUBLE = 1
    """Field type double."""

    TYPE_FLOAT = 2
    """Field type float."""

    TYPE_INT64 = 3
    """Field type int64."""

    TYPE_UINT64 = 4
    """Field type uint64."""

    TYPE_INT32 = 5
    """Field type int32."""

    TYPE_FIXED64 = 6
    """Field type fixed64."""

    TYPE_FIXED32 = 7
    """Field type fixed32."""

    TYPE_BOOL = 8
    """Field type bool."""

    TYPE_STRING = 9
    """Field type string."""

    TYPE_GROUP = 10
    """Field type group. Proto2 syntax only, and deprecated."""

    TYPE_MESSAGE = 11
    """Field type message."""

    TYPE_BYTES = 12
    """Field type bytes."""

    TYPE_UINT32 = 13
    """Field type uint32."""

    TYPE_ENUM = 14
    """Field type enum."""

    TYPE_SFIXED32 = 15
    """Field type sfixed32."""

    TYPE_SFIXED64 = 16
    """Field type sfixed64."""

    TYPE_SINT32 = 17
    """Field type sint32."""

    TYPE_SINT64 = 18
    """Field type sint64."""


class FieldCardinality(aristaproto.Enum):
    """Whether a field is optional, required, or repeated."""

    CARDINALITY_UNKNOWN = 0
    """For fields with unknown cardinality."""

    CARDINALITY_OPTIONAL = 1
    """For optional fields."""

    CARDINALITY_REQUIRED = 2
    """For required fields. Proto2 syntax only."""

    CARDINALITY_REPEATED = 3
    """For repeated fields."""


class Edition(aristaproto.Enum):
    """The full set of known editions."""

    UNKNOWN = 0
    """A placeholder for an unknown edition value."""

    PROTO2 = 998
    """
    Legacy syntax "editions".  These pre-date editions, but behave much like
     distinct editions.  These can't be used to specify the edition of proto
     files, but feature definitions must supply proto2/proto3 defaults for
     backwards compatibility.
    """

    PROTO3 = 999
    _2023 = 1000
    """
    Editions that have been released.  The specific values are arbitrary and
     should not be depended on, but they will always be time-ordered for easy
     comparison.
    """

    _2024 = 1001
    _1_TEST_ONLY = 1
    """
    Placeholder editions for testing feature resolution.  These should not be
     used or relyed on outside of tests.
    """

    _2_TEST_ONLY = 2
    _99997_TEST_ONLY = 99997
    _99998_TEST_ONLY = 99998
    _99999_TEST_ONLY = 99999
    MAX = 2147483647
    """
    Placeholder for specifying unbounded edition support.  This should only
     ever be used by plugins that can expect to never require any changes to
     support a new edition.
    """


class ExtensionRangeOptionsVerificationState(aristaproto.Enum):
    """The verification state of the extension range."""

    DECLARATION = 0
    """All the extensions of the range must be declared."""

    UNVERIFIED = 1


class FieldDescriptorProtoType(aristaproto.Enum):
    TYPE_DOUBLE = 1
    """
    0 is reserved for errors.
     Order is weird for historical reasons.
    """

    TYPE_FLOAT = 2
    TYPE_INT64 = 3
    """
    Not ZigZag encoded.  Negative numbers take 10 bytes.  Use TYPE_SINT64 if
     negative values are likely.
    """

    TYPE_UINT64 = 4
    TYPE_INT32 = 5
    """
    Not ZigZag encoded.  Negative numbers take 10 bytes.  Use TYPE_SINT32 if
     negative values are likely.
    """

    TYPE_FIXED64 = 6
    TYPE_FIXED32 = 7
    TYPE_BOOL = 8
    TYPE_STRING = 9
    TYPE_GROUP = 10
    """
    Tag-delimited aggregate.
     Group type is deprecated and not supported after google.protobuf. However, Proto3
     implementations should still be able to parse the group wire format and
     treat group fields as unknown fields.  In Editions, the group wire format
     can be enabled via the `message_encoding` feature.
    """

    TYPE_MESSAGE = 11
    TYPE_BYTES = 12
    """New in version 2."""

    TYPE_UINT32 = 13
    TYPE_ENUM = 14
    TYPE_SFIXED32 = 15
    TYPE_SFIXED64 = 16
    TYPE_SINT32 = 17
    TYPE_SINT64 = 18


class FieldDescriptorProtoLabel(aristaproto.Enum):
    LABEL_OPTIONAL = 1
    """0 is reserved for errors"""

    LABEL_REPEATED = 3
    LABEL_REQUIRED = 2
    """
    The required label is only allowed in google.protobuf.  In proto3 and Editions
     it's explicitly prohibited.  In Editions, the `field_presence` feature
     can be used to get this behavior.
    """


class FileOptionsOptimizeMode(aristaproto.Enum):
    """Generated classes can be optimized for speed or code size."""

    SPEED = 1
    CODE_SIZE = 2
    """etc."""

    LITE_RUNTIME = 3


class FieldOptionsCType(aristaproto.Enum):
    STRING = 0
    """Default mode."""

    CORD = 1
    """
    The option [ctype=CORD] may be applied to a non-repeated field of type
     "bytes". It indicates that in C++, the data should be stored in a Cord
     instead of a string.  For very large strings, this may reduce memory
     fragmentation. It may also allow better performance when parsing from a
     Cord, or when parsing with aliasing enabled, as the parsed Cord may then
     alias the original buffer.
    """

    STRING_PIECE = 2


class FieldOptionsJsType(aristaproto.Enum):
    JS_NORMAL = 0
    """Use the default type."""

    JS_STRING = 1
    """Use JavaScript strings."""

    JS_NUMBER = 2
    """Use JavaScript numbers."""


class FieldOptionsOptionRetention(aristaproto.Enum):
    """
    If set to RETENTION_SOURCE, the option will be omitted from the binary.
     Note: as of January 2023, support for this is in progress and does not yet
     have an effect (b/264593489).
    """

    RETENTION_UNKNOWN = 0
    RETENTION_RUNTIME = 1
    RETENTION_SOURCE = 2


class FieldOptionsOptionTargetType(aristaproto.Enum):
    """
    This indicates the types of entities that the field may apply to when used
     as an option. If it is unset, then the field may be freely used as an
     option on any kind of entity. Note: as of January 2023, support for this is
     in progress and does not yet have an effect (b/264593489).
    """

    TARGET_TYPE_UNKNOWN = 0
    TARGET_TYPE_FILE = 1
    TARGET_TYPE_EXTENSION_RANGE = 2
    TARGET_TYPE_MESSAGE = 3
    TARGET_TYPE_FIELD = 4
    TARGET_TYPE_ONEOF = 5
    TARGET_TYPE_ENUM = 6
    TARGET_TYPE_ENUM_ENTRY = 7
    TARGET_TYPE_SERVICE = 8
    TARGET_TYPE_METHOD = 9


class MethodOptionsIdempotencyLevel(aristaproto.Enum):
    """
    Is this method side-effect-free (or safe in HTTP parlance), or idempotent,
     or neither? HTTP based RPC implementation may choose GET verb for safe
     methods, and PUT verb for idempotent methods instead of the default POST.
    """

    IDEMPOTENCY_UNKNOWN = 0
    NO_SIDE_EFFECTS = 1
    IDEMPOTENT = 2


class FeatureSetFieldPresence(aristaproto.Enum):
    FIELD_PRESENCE_UNKNOWN = 0
    EXPLICIT = 1
    IMPLICIT = 2
    LEGACY_REQUIRED = 3


class FeatureSetEnumType(aristaproto.Enum):
    ENUM_TYPE_UNKNOWN = 0
    OPEN = 1
    CLOSED = 2


class FeatureSetRepeatedFieldEncoding(aristaproto.Enum):
    REPEATED_FIELD_ENCODING_UNKNOWN = 0
    PACKED = 1
    EXPANDED = 2


class FeatureSetUtf8Validation(aristaproto.Enum):
    UTF8_VALIDATION_UNKNOWN = 0
    VERIFY = 2
    NONE = 3


class FeatureSetMessageEncoding(aristaproto.Enum):
    MESSAGE_ENCODING_UNKNOWN = 0
    LENGTH_PREFIXED = 1
    DELIMITED = 2


class FeatureSetJsonFormat(aristaproto.Enum):
    JSON_FORMAT_UNKNOWN = 0
    ALLOW = 1
    LEGACY_BEST_EFFORT = 2


class GeneratedCodeInfoAnnotationSemantic(aristaproto.Enum):
    """
    Represents the identified object's effect on the element in the original
     .proto file.
    """

    NONE = 0
    """There is no effect or the effect is indescribable."""

    SET = 1
    """The element is set or otherwise mutated."""

    ALIAS = 2
    """An alias to the element is returned."""


class NullValue(aristaproto.Enum):
    """
    `NullValue` is a singleton enumeration to represent the null value for the
     `Value` type union.

     The JSON representation for `NullValue` is JSON `null`.
    """

    _ = 0
    """Null value."""


@dataclass(eq=False, repr=False)
class Any(aristaproto.Message):
    """
    `Any` contains an arbitrary serialized protocol buffer message along with a
     URL that describes the type of the serialized message.

     Protobuf library provides support to pack/unpack Any values in the form
     of utility functions or additional generated methods of the Any type.

     Example 1: Pack and unpack a message in C++.

         Foo foo = ...;
         Any any;
         any.PackFrom(foo);
         ...
         if (any.UnpackTo(&foo)) {
           ...
         }

     Example 2: Pack and unpack a message in Java.

         Foo foo = ...;
         Any any = Any.pack(foo);
         ...
         if (any.is(Foo.class)) {
           foo = any.unpack(Foo.class);
         }
         // or ...
         if (any.isSameTypeAs(Foo.getDefaultInstance())) {
           foo = any.unpack(Foo.getDefaultInstance());
         }

      Example 3: Pack and unpack a message in Python.

         foo = Foo(...)
         any = Any()
         any.Pack(foo)
         ...
         if any.Is(Foo.DESCRIPTOR):
           any.Unpack(foo)
           ...

      Example 4: Pack and unpack a message in Go

          foo := &pb.Foo{...}
          any, err := anypb.New(foo)
          if err != nil {
            ...
          }
          ...
          foo := &pb.Foo{}
          if err := any.UnmarshalTo(foo); err != nil {
            ...
          }

     The pack methods provided by protobuf library will by default use
     'type.googleapis.com/full.type.name' as the type URL and the unpack
     methods only use the fully qualified type name after the last '/'
     in the type URL, for example "foo.bar.com/x/y.z" will yield type
     name "y.z".

     JSON
     ====
     The JSON representation of an `Any` value uses the regular
     representation of the deserialized, embedded message, with an
     additional field `@type` which contains the type URL. Example:

         package google.profile;
         message Person {
           string first_name = 1;
           string last_name = 2;
         }

         {
           "@type": "type.googleapis.com/google.profile.Person",
           "firstName": <string>,
           "lastName": <string>
         }

     If the embedded message type is well-known and has a custom JSON
     representation, that representation will be embedded adding a field
     `value` which holds the custom JSON in addition to the `@type`
     field. Example (for message [google.protobuf.Duration][]):

         {
           "@type": "type.googleapis.com/google.protobuf.Duration",
           "value": "1.212s"
         }
    """

    type_url: str = aristaproto.string_field(1)
    """
    A URL/resource name that uniquely identifies the type of the serialized
     protocol buffer message. This string must contain at least
     one "/" character. The last segment of the URL's path must represent
     the fully qualified name of the type (as in
     `path/google.protobuf.Duration`). The name should be in a canonical form
     (e.g., leading "." is not accepted).

     In practice, teams usually precompile into the binary all types that they
     expect it to use in the context of Any. However, for URLs which use the
     scheme `http`, `https`, or no scheme, one can optionally set up a type
     server that maps type URLs to message definitions as follows:

     * If no scheme is provided, `https` is assumed.
     * An HTTP GET on the URL must yield a [google.protobuf.Type][]
       value in binary format, or produce an error.
     * Applications are allowed to cache lookup results based on the
       URL, or have them precompiled into a binary to avoid any
       lookup. Therefore, binary compatibility needs to be preserved
       on changes to types. (Use versioned type names to manage
       breaking changes.)

     Note: this functionality is not currently available in the official
     protobuf release, and it is not used for type URLs beginning with
     type.googleapis.com. As of May 2023, there are no widely used type server
     implementations and no plans to implement one.

     Schemes other than `http`, `https` (or the empty scheme) might be
     used with implementation specific semantics.
    """

    value: bytes = aristaproto.bytes_field(2)
    """
    Must be a valid serialized protocol buffer of the above specified type.
    """


@dataclass(eq=False, repr=False)
class SourceContext(aristaproto.Message):
    """
    `SourceContext` represents information about the source of a
     protobuf element, like the file in which it is defined.
    """

    file_name: str = aristaproto.string_field(1)
    """
    The path-qualified name of the .proto file that contained the associated
     protobuf element.  For example: `"google/protobuf/source_context.proto"`.
    """


@dataclass(eq=False, repr=False)
class Type(aristaproto.Message):
    """A protocol buffer message type."""

    name: str = aristaproto.string_field(1)
    """The fully qualified message name."""

    fields: List["Field"] = aristaproto.message_field(2)
    """The list of fields."""

    oneofs: List[str] = aristaproto.string_field(3)
    """The list of types appearing in `oneof` definitions in this type."""

    options: List["Option"] = aristaproto.message_field(4)
    """The protocol buffer options."""

    source_context: "SourceContext" = aristaproto.message_field(5)
    """The source context."""

    syntax: "Syntax" = aristaproto.enum_field(6)
    """The source syntax."""

    edition: str = aristaproto.string_field(7)
    """
    The source edition string, only valid when syntax is SYNTAX_EDITIONS.
    """


@dataclass(eq=False, repr=False)
class Field(aristaproto.Message):
    """A single field of a message type."""

    kind: "FieldKind" = aristaproto.enum_field(1)
    """The field type."""

    cardinality: "FieldCardinality" = aristaproto.enum_field(2)
    """The field cardinality."""

    number: int = aristaproto.int32_field(3)
    """The field number."""

    name: str = aristaproto.string_field(4)
    """The field name."""

    type_url: str = aristaproto.string_field(6)
    """
    The field type URL, without the scheme, for message or enumeration
     types. Example: `"type.googleapis.com/google.protobuf.Timestamp"`.
    """

    oneof_index: int = aristaproto.int32_field(7)
    """
    The index of the field type in `Type.oneofs`, for message or enumeration
     types. The first type has index 1; zero means the type is not in the list.
    """

    packed: bool = aristaproto.bool_field(8)
    """Whether to use alternative packed wire representation."""

    options: List["Option"] = aristaproto.message_field(9)
    """The protocol buffer options."""

    json_name: str = aristaproto.string_field(10)
    """The field JSON name."""

    default_value: str = aristaproto.string_field(11)
    """
    The string value of the default value of this field. Proto2 syntax only.
    """


@dataclass(eq=False, repr=False)
class Enum(aristaproto.Message):
    """Enum type definition."""

    name: str = aristaproto.string_field(1)
    """Enum type name."""

    enumvalue: List["EnumValue"] = aristaproto.message_field(
        2, wraps=aristaproto.TYPE_ENUM
    )
    """Enum value definitions."""

    options: List["Option"] = aristaproto.message_field(3)
    """Protocol buffer options."""

    source_context: "SourceContext" = aristaproto.message_field(4)
    """The source context."""

    syntax: "Syntax" = aristaproto.enum_field(5)
    """The source syntax."""

    edition: str = aristaproto.string_field(6)
    """
    The source edition string, only valid when syntax is SYNTAX_EDITIONS.
    """


@dataclass(eq=False, repr=False)
class EnumValue(aristaproto.Message):
    """Enum value definition."""

    name: str = aristaproto.string_field(1)
    """Enum value name."""

    number: int = aristaproto.int32_field(2)
    """Enum value number."""

    options: List["Option"] = aristaproto.message_field(3)
    """Protocol buffer options."""


@dataclass(eq=False, repr=False)
class Option(aristaproto.Message):
    """
    A protocol buffer option, which can be attached to a message, field,
     enumeration, etc.
    """

    name: str = aristaproto.string_field(1)
    """
    The option's name. For protobuf built-in options (options defined in
     descriptor.proto), this is the short name. For example, `"map_entry"`.
     For custom options, it should be the fully-qualified name. For example,
     `"google.api.http"`.
    """

    value: "Any" = aristaproto.message_field(2)
    """
    The option's value packed in an Any message. If the value is a primitive,
     the corresponding wrapper type defined in google/protobuf/wrappers.proto
     should be used. If the value is an enum, it should be stored as an int32
     value using the google.protobuf.Int32Value type.
    """


@dataclass(eq=False, repr=False)
class Api(aristaproto.Message):
    """
    Api is a light-weight descriptor for an API Interface.

     Interfaces are also described as "protocol buffer services" in some contexts,
     such as by the "service" keyword in a .proto file, but they are different
     from API Services, which represent a concrete implementation of an interface
     as opposed to simply a description of methods and bindings. They are also
     sometimes simply referred to as "APIs" in other contexts, such as the name of
     this message itself. See https://cloud.google.com/apis/design/glossary for
     detailed terminology.
    """

    name: str = aristaproto.string_field(1)
    """
    The fully qualified name of this interface, including package name
     followed by the interface's simple name.
    """

    methods: List["Method"] = aristaproto.message_field(2)
    """The methods of this interface, in unspecified order."""

    options: List["Option"] = aristaproto.message_field(3)
    """Any metadata attached to the interface."""

    version: str = aristaproto.string_field(4)
    """
    A version string for this interface. If specified, must have the form
     `major-version.minor-version`, as in `1.10`. If the minor version is
     omitted, it defaults to zero. If the entire version field is empty, the
     major version is derived from the package name, as outlined below. If the
     field is not empty, the version in the package name will be verified to be
     consistent with what is provided here.

     The versioning schema uses [semantic
     versioning](http://semver.org) where the major version number
     indicates a breaking change and the minor version an additive,
     non-breaking change. Both version numbers are signals to users
     what to expect from different versions, and should be carefully
     chosen based on the product plan.

     The major version is also reflected in the package name of the
     interface, which must end in `v<major-version>`, as in
     `google.feature.v1`. For major versions 0 and 1, the suffix can
     be omitted. Zero major versions must only be used for
     experimental, non-GA interfaces.
    """

    source_context: "SourceContext" = aristaproto.message_field(5)
    """
    Source context for the protocol buffer service represented by this
     message.
    """

    mixins: List["Mixin"] = aristaproto.message_field(6)
    """Included interfaces. See [Mixin][]."""

    syntax: "Syntax" = aristaproto.enum_field(7)
    """The source syntax of the service."""


@dataclass(eq=False, repr=False)
class Method(aristaproto.Message):
    """Method represents a method of an API interface."""

    name: str = aristaproto.string_field(1)
    """The simple name of this method."""

    request_type_url: str = aristaproto.string_field(2)
    """A URL of the input message type."""

    request_streaming: bool = aristaproto.bool_field(3)
    """If true, the request is streamed."""

    response_type_url: str = aristaproto.string_field(4)
    """The URL of the output message type."""

    response_streaming: bool = aristaproto.bool_field(5)
    """If true, the response is streamed."""

    options: List["Option"] = aristaproto.message_field(6)
    """Any metadata attached to the method."""

    syntax: "Syntax" = aristaproto.enum_field(7)
    """The source syntax of this method."""


@dataclass(eq=False, repr=False)
class Mixin(aristaproto.Message):
    """
    Declares an API Interface to be included in this interface. The including
     interface must redeclare all the methods from the included interface, but
     documentation and options are inherited as follows:

     - If after comment and whitespace stripping, the documentation
       string of the redeclared method is empty, it will be inherited
       from the original method.

     - Each annotation belonging to the service config (http,
       visibility) which is not set in the redeclared method will be
       inherited.

     - If an http annotation is inherited, the path pattern will be
       modified as follows. Any version prefix will be replaced by the
       version of the including interface plus the [root][] path if
       specified.

     Example of a simple mixin:

         package google.acl.v1;
         service AccessControl {
           // Get the underlying ACL object.
           rpc GetAcl(GetAclRequest) returns (Acl) {
             option (google.api.http).get = "/v1/{resource=**}:getAcl";
           }
         }

         package google.storage.v2;
         service Storage {
           rpc GetAcl(GetAclRequest) returns (Acl);

           // Get a data record.
           rpc GetData(GetDataRequest) returns (Data) {
             option (google.api.http).get = "/v2/{resource=**}";
           }
         }

     Example of a mixin configuration:

         apis:
         - name: google.storage.v2.Storage
           mixins:
           - name: google.acl.v1.AccessControl

     The mixin construct implies that all methods in `AccessControl` are
     also declared with same name and request/response types in
     `Storage`. A documentation generator or annotation processor will
     see the effective `Storage.GetAcl` method after inherting
     documentation and annotations as follows:

         service Storage {
           // Get the underlying ACL object.
           rpc GetAcl(GetAclRequest) returns (Acl) {
             option (google.api.http).get = "/v2/{resource=**}:getAcl";
           }
           ...
         }

     Note how the version in the path pattern changed from `v1` to `v2`.

     If the `root` field in the mixin is specified, it should be a
     relative path under which inherited HTTP paths are placed. Example:

         apis:
         - name: google.storage.v2.Storage
           mixins:
           - name: google.acl.v1.AccessControl
             root: acls

     This implies the following inherited HTTP annotation:

         service Storage {
           // Get the underlying ACL object.
           rpc GetAcl(GetAclRequest) returns (Acl) {
             option (google.api.http).get = "/v2/acls/{resource=**}:getAcl";
           }
           ...
         }
    """

    name: str = aristaproto.string_field(1)
    """The fully qualified name of the interface which is included."""

    root: str = aristaproto.string_field(2)
    """
    If non-empty specifies a path under which inherited HTTP paths
     are rooted.
    """


@dataclass(eq=False, repr=False)
class FileDescriptorSet(aristaproto.Message):
    """
    The protocol compiler can output a FileDescriptorSet containing the .proto
     files it parses.
    """

    file: List["FileDescriptorProto"] = aristaproto.message_field(1)


@dataclass(eq=False, repr=False)
class FileDescriptorProto(aristaproto.Message):
    """Describes a complete .proto file."""

    name: str = aristaproto.string_field(1)
    package: str = aristaproto.string_field(2)
    dependency: List[str] = aristaproto.string_field(3)
    """Names of files imported by this file."""

    public_dependency: List[int] = aristaproto.int32_field(10)
    """Indexes of the public imported files in the dependency list above."""

    weak_dependency: List[int] = aristaproto.int32_field(11)
    """
    Indexes of the weak imported files in the dependency list.
     For Google-internal migration only. Do not use.
    """

    message_type: List["DescriptorProto"] = aristaproto.message_field(4)
    """All top-level definitions in this file."""

    enum_type: List["EnumDescriptorProto"] = aristaproto.message_field(5)
    service: List["ServiceDescriptorProto"] = aristaproto.message_field(6)
    extension: List["FieldDescriptorProto"] = aristaproto.message_field(7)
    options: "FileOptions" = aristaproto.message_field(8)
    source_code_info: "SourceCodeInfo" = aristaproto.message_field(9)
    """
    This field contains optional information about the original source code.
     You may safely remove this entire field without harming runtime
     functionality of the descriptors -- the information is needed only by
     development tools.
    """

    syntax: str = aristaproto.string_field(12)
    """
    The syntax of the proto file.
     The supported values are "proto2", "proto3", and "editions".

     If `edition` is present, this value must be "editions".
    """

    edition: "Edition" = aristaproto.enum_field(14)
    """The edition of the proto file."""


@dataclass(eq=False, repr=False)
class DescriptorProto(aristaproto.Message):
    """Describes a message type."""

    name: str = aristaproto.string_field(1)
    field: List["FieldDescriptorProto"] = aristaproto.message_field(2)
    extension: List["FieldDescriptorProto"] = aristaproto.message_field(6)
    nested_type: List["DescriptorProto"] = aristaproto.message_field(3)
    enum_type: List["EnumDescriptorProto"] = aristaproto.message_field(4)
    extension_range: List["DescriptorProtoExtensionRange"] = aristaproto.message_field(
        5
    )
    oneof_decl: List["OneofDescriptorProto"] = aristaproto.message_field(8)
    options: "MessageOptions" = aristaproto.message_field(7)
    reserved_range: List["DescriptorProtoReservedRange"] = aristaproto.message_field(9)
    reserved_name: List[str] = aristaproto.string_field(10)
    """
    Reserved field names, which may not be used by fields in the same message.
     A given name may only be reserved once.
    """


@dataclass(eq=False, repr=False)
class DescriptorProtoExtensionRange(aristaproto.Message):
    start: int = aristaproto.int32_field(1)
    end: int = aristaproto.int32_field(2)
    options: "ExtensionRangeOptions" = aristaproto.message_field(3)


@dataclass(eq=False, repr=False)
class DescriptorProtoReservedRange(aristaproto.Message):
    """
    Range of reserved tag numbers. Reserved tag numbers may not be used by
     fields or extension ranges in the same message. Reserved ranges may
     not overlap.
    """

    start: int = aristaproto.int32_field(1)
    end: int = aristaproto.int32_field(2)


@dataclass(eq=False, repr=False)
class ExtensionRangeOptions(aristaproto.Message):
    uninterpreted_option: List["UninterpretedOption"] = aristaproto.message_field(999)
    """The parser stores options it doesn't recognize here. See above."""

    declaration: List["ExtensionRangeOptionsDeclaration"] = aristaproto.message_field(2)
    """
    For external users: DO NOT USE. We are in the process of open sourcing
     extension declaration and executing internal cleanups before it can be
     used externally.
    """

    features: "FeatureSet" = aristaproto.message_field(50)
    """Any features defined in the specific edition."""

    verification: "ExtensionRangeOptionsVerificationState" = aristaproto.enum_field(3)
    """
    The verification state of the range.
     TODO: flip the default to DECLARATION once all empty ranges
     are marked as UNVERIFIED.
    """


@dataclass(eq=False, repr=False)
class ExtensionRangeOptionsDeclaration(aristaproto.Message):
    number: int = aristaproto.int32_field(1)
    """The extension number declared within the extension range."""

    full_name: str = aristaproto.string_field(2)
    """
    The fully-qualified name of the extension field. There must be a leading
     dot in front of the full name.
    """

    type: str = aristaproto.string_field(3)
    """
    The fully-qualified type name of the extension field. Unlike
     Metadata.type, Declaration.type must have a leading dot for messages
     and enums.
    """

    reserved: bool = aristaproto.bool_field(5)
    """
    If true, indicates that the number is reserved in the extension range,
     and any extension field with the number will fail to compile. Set this
     when a declared extension field is deleted.
    """

    repeated: bool = aristaproto.bool_field(6)
    """
    If true, indicates that the extension must be defined as repeated.
     Otherwise the extension must be defined as optional.
    """


@dataclass(eq=False, repr=False)
class FieldDescriptorProto(aristaproto.Message):
    """Describes a field within a message."""

    name: str = aristaproto.string_field(1)
    number: int = aristaproto.int32_field(3)
    label: "FieldDescriptorProtoLabel" = aristaproto.enum_field(4)
    type: "FieldDescriptorProtoType" = aristaproto.enum_field(5)
    """
    If type_name is set, this need not be set.  If both this and type_name
     are set, this must be one of TYPE_ENUM, TYPE_MESSAGE or TYPE_GROUP.
    """

    type_name: str = aristaproto.string_field(6)
    """
    For message and enum types, this is the name of the type.  If the name
     starts with a '.', it is fully-qualified.  Otherwise, C++-like scoping
     rules are used to find the type (i.e. first the nested types within this
     message are searched, then within the parent, on up to the root
     namespace).
    """

    extendee: str = aristaproto.string_field(2)
    """
    For extensions, this is the name of the type being extended.  It is
     resolved in the same manner as type_name.
    """

    default_value: str = aristaproto.string_field(7)
    """
    For numeric types, contains the original text representation of the value.
     For booleans, "true" or "false".
     For strings, contains the default text contents (not escaped in any way).
     For bytes, contains the C escaped value.  All bytes >= 128 are escaped.
    """

    oneof_index: int = aristaproto.int32_field(9)
    """
    If set, gives the index of a oneof in the containing type's oneof_decl
     list.  This field is a member of that oneof.
    """

    json_name: str = aristaproto.string_field(10)
    """
    JSON name of this field. The value is set by protocol compiler. If the
     user has set a "json_name" option on this field, that option's value
     will be used. Otherwise, it's deduced from the field's name by converting
     it to camelCase.
    """

    options: "FieldOptions" = aristaproto.message_field(8)
    proto3_optional: bool = aristaproto.bool_field(17)
    """
    If true, this is a proto3 "optional". When a proto3 field is optional, it
     tracks presence regardless of field type.

     When proto3_optional is true, this field must belong to a oneof to signal
     to old proto3 clients that presence is tracked for this field. This oneof
     is known as a "synthetic" oneof, and this field must be its sole member
     (each proto3 optional field gets its own synthetic oneof). Synthetic oneofs
     exist in the descriptor only, and do not generate any API. Synthetic oneofs
     must be ordered after all "real" oneofs.

     For message fields, proto3_optional doesn't create any semantic change,
     since non-repeated message fields always track presence. However it still
     indicates the semantic detail of whether the user wrote "optional" or not.
     This can be useful for round-tripping the .proto file. For consistency we
     give message fields a synthetic oneof also, even though it is not required
     to track presence. This is especially important because the parser can't
     tell if a field is a message or an enum, so it must always create a
     synthetic oneof.

     Proto2 optional fields do not set this flag, because they already indicate
     optional with `LABEL_OPTIONAL`.
    """


@dataclass(eq=False, repr=False)
class OneofDescriptorProto(aristaproto.Message):
    """Describes a oneof."""

    name: str = aristaproto.string_field(1)
    options: "OneofOptions" = aristaproto.message_field(2)


@dataclass(eq=False, repr=False)
class EnumDescriptorProto(aristaproto.Message):
    """Describes an enum type."""

    name: str = aristaproto.string_field(1)
    value: List["EnumValueDescriptorProto"] = aristaproto.message_field(2)
    options: "EnumOptions" = aristaproto.message_field(3)
    reserved_range: List[
        "EnumDescriptorProtoEnumReservedRange"
    ] = aristaproto.message_field(4)
    """
    Range of reserved numeric values. Reserved numeric values may not be used
     by enum values in the same enum declaration. Reserved ranges may not
     overlap.
    """

    reserved_name: List[str] = aristaproto.string_field(5)
    """
    Reserved enum value names, which may not be reused. A given name may only
     be reserved once.
    """


@dataclass(eq=False, repr=False)
class EnumDescriptorProtoEnumReservedRange(aristaproto.Message):
    """
    Range of reserved numeric values. Reserved values may not be used by
     entries in the same enum. Reserved ranges may not overlap.

     Note that this is distinct from DescriptorProto.ReservedRange in that it
     is inclusive such that it can appropriately represent the entire int32
     domain.
    """

    start: int = aristaproto.int32_field(1)
    end: int = aristaproto.int32_field(2)


@dataclass(eq=False, repr=False)
class EnumValueDescriptorProto(aristaproto.Message):
    """Describes a value within an enum."""

    name: str = aristaproto.string_field(1)
    number: int = aristaproto.int32_field(2)
    options: "EnumValueOptions" = aristaproto.message_field(3)


@dataclass(eq=False, repr=False)
class ServiceDescriptorProto(aristaproto.Message):
    """Describes a service."""

    name: str = aristaproto.string_field(1)
    method: List["MethodDescriptorProto"] = aristaproto.message_field(2)
    options: "ServiceOptions" = aristaproto.message_field(3)


@dataclass(eq=False, repr=False)
class MethodDescriptorProto(aristaproto.Message):
    """Describes a method of a service."""

    name: str = aristaproto.string_field(1)
    input_type: str = aristaproto.string_field(2)
    """
    Input and output type names.  These are resolved in the same way as
     FieldDescriptorProto.type_name, but must refer to a message type.
    """

    output_type: str = aristaproto.string_field(3)
    options: "MethodOptions" = aristaproto.message_field(4)
    client_streaming: bool = aristaproto.bool_field(5)
    """Identifies if client streams multiple client messages"""

    server_streaming: bool = aristaproto.bool_field(6)
    """Identifies if server streams multiple server messages"""


@dataclass(eq=False, repr=False)
class FileOptions(aristaproto.Message):
    java_package: str = aristaproto.string_field(1)
    """
    Sets the Java package where classes generated from this .proto will be
     placed.  By default, the proto package is used, but this is often
     inappropriate because proto packages do not normally start with backwards
     domain names.
    """

    java_outer_classname: str = aristaproto.string_field(8)
    """
    Controls the name of the wrapper Java class generated for the .proto file.
     That class will always contain the .proto file's getDescriptor() method as
     well as any top-level extensions defined in the .proto file.
     If java_multiple_files is disabled, then all the other classes from the
     .proto file will be nested inside the single wrapper outer class.
    """

    java_multiple_files: bool = aristaproto.bool_field(10)
    """
    If enabled, then the Java code generator will generate a separate .java
     file for each top-level message, enum, and service defined in the .proto
     file.  Thus, these types will *not* be nested inside the wrapper class
     named by java_outer_classname.  However, the wrapper class will still be
     generated to contain the file's getDescriptor() method as well as any
     top-level extensions defined in the file.
    """

    java_generate_equals_and_hash: bool = aristaproto.bool_field(20)
    """This option does nothing."""

    java_string_check_utf8: bool = aristaproto.bool_field(27)
    """
    A proto2 file can set this to true to opt in to UTF-8 checking for Java,
     which will throw an exception if invalid UTF-8 is parsed from the wire or
     assigned to a string field.

     TODO: clarify exactly what kinds of field types this option
     applies to, and update these docs accordingly.

     Proto3 files already perform these checks. Setting the option explicitly to
     false has no effect: it cannot be used to opt proto3 files out of UTF-8
     checks.
    """

    optimize_for: "FileOptionsOptimizeMode" = aristaproto.enum_field(9)
    go_package: str = aristaproto.string_field(11)
    """
    Sets the Go package where structs generated from this .proto will be
     placed. If omitted, the Go package will be derived from the following:
       - The basename of the package import path, if provided.
       - Otherwise, the package statement in the .proto file, if present.
       - Otherwise, the basename of the .proto file, without extension.
    """

    cc_generic_services: bool = aristaproto.bool_field(16)
    """
    Should generic services be generated in each language?  "Generic" services
     are not specific to any particular RPC system.  They are generated by the
     main code generators in each language (without additional plugins).
     Generic services were the only kind of service generation supported by
     early versions of google.protobuf.

     Generic services are now considered deprecated in favor of using plugins
     that generate code specific to your particular RPC system.  Therefore,
     these default to false.  Old code which depends on generic services should
     explicitly set them to true.
    """

    java_generic_services: bool = aristaproto.bool_field(17)
    py_generic_services: bool = aristaproto.bool_field(18)
    deprecated: bool = aristaproto.bool_field(23)
    """
    Is this file deprecated?
     Depending on the target platform, this can emit Deprecated annotations
     for everything in the file, or it will be completely ignored; in the very
     least, this is a formalization for deprecating files.
    """

    cc_enable_arenas: bool = aristaproto.bool_field(31)
    """
    Enables the use of arenas for the proto messages in this file. This applies
     only to generated classes for C++.
    """

    objc_class_prefix: str = aristaproto.string_field(36)
    """
    Sets the objective c class prefix which is prepended to all objective c
     generated classes from this .proto. There is no default.
    """

    csharp_namespace: str = aristaproto.string_field(37)
    """Namespace for generated classes; defaults to the package."""

    swift_prefix: str = aristaproto.string_field(39)
    """
    By default Swift generators will take the proto package and CamelCase it
     replacing '.' with underscore and use that to prefix the types/symbols
     defined. When this options is provided, they will use this value instead
     to prefix the types/symbols defined.
    """

    php_class_prefix: str = aristaproto.string_field(40)
    """
    Sets the php class prefix which is prepended to all php generated classes
     from this .proto. Default is empty.
    """

    php_namespace: str = aristaproto.string_field(41)
    """
    Use this option to change the namespace of php generated classes. Default
     is empty. When this option is empty, the package name will be used for
     determining the namespace.
    """

    php_metadata_namespace: str = aristaproto.string_field(44)
    """
    Use this option to change the namespace of php generated metadata classes.
     Default is empty. When this option is empty, the proto file name will be
     used for determining the namespace.
    """

    ruby_package: str = aristaproto.string_field(45)
    """
    Use this option to change the package of ruby generated classes. Default
     is empty. When this option is not set, the package name will be used for
     determining the ruby package.
    """

    features: "FeatureSet" = aristaproto.message_field(50)
    """Any features defined in the specific edition."""

    uninterpreted_option: List["UninterpretedOption"] = aristaproto.message_field(999)
    """
    The parser stores options it doesn't recognize here.
     See the documentation for the "Options" section above.
    """

    def __post_init__(self) -> None:
        super().__post_init__()
        if self.is_set("java_generate_equals_and_hash"):
            warnings.warn(
                "FileOptions.java_generate_equals_and_hash is deprecated",
                DeprecationWarning,
            )


@dataclass(eq=False, repr=False)
class MessageOptions(aristaproto.Message):
    message_set_wire_format: bool = aristaproto.bool_field(1)
    """
    Set true to use the old proto1 MessageSet wire format for extensions.
     This is provided for backwards-compatibility with the MessageSet wire
     format.  You should not use this for any other reason:  It's less
     efficient, has fewer features, and is more complicated.

     The message must be defined exactly as follows:
       message Foo {
         option message_set_wire_format = true;
         extensions 4 to max;
       }
     Note that the message cannot have any defined fields; MessageSets only
     have extensions.

     All extensions of your type must be singular messages; e.g. they cannot
     be int32s, enums, or repeated messages.

     Because this is an option, the above two restrictions are not enforced by
     the protocol compiler.
    """

    no_standard_descriptor_accessor: bool = aristaproto.bool_field(2)
    """
    Disables the generation of the standard "descriptor()" accessor, which can
     conflict with a field of the same name.  This is meant to make migration
     from proto1 easier; new code should avoid fields named "descriptor".
    """

    deprecated: bool = aristaproto.bool_field(3)
    """
    Is this message deprecated?
     Depending on the target platform, this can emit Deprecated annotations
     for the message, or it will be completely ignored; in the very least,
     this is a formalization for deprecating messages.
    """

    map_entry: bool = aristaproto.bool_field(7)
    """
    Whether the message is an automatically generated map entry type for the
     maps field.

     For maps fields:
         map<KeyType, ValueType> map_field = 1;
     The parsed descriptor looks like:
         message MapFieldEntry {
             option map_entry = true;
             optional KeyType key = 1;
             optional ValueType value = 2;
         }
         repeated MapFieldEntry map_field = 1;

     Implementations may choose not to generate the map_entry=true message, but
     use a native map in the target language to hold the keys and values.
     The reflection APIs in such implementations still need to work as
     if the field is a repeated message field.

     NOTE: Do not set the option in .proto files. Always use the maps syntax
     instead. The option should only be implicitly set by the proto compiler
     parser.
    """

    deprecated_legacy_json_field_conflicts: bool = aristaproto.bool_field(11)
    """
    Enable the legacy handling of JSON field name conflicts.  This lowercases
     and strips underscored from the fields before comparison in proto3 only.
     The new behavior takes `json_name` into account and applies to proto2 as
     well.

     This should only be used as a temporary measure against broken builds due
     to the change in behavior for JSON field name conflicts.

     TODO This is legacy behavior we plan to remove once downstream
     teams have had time to migrate.
    """

    features: "FeatureSet" = aristaproto.message_field(12)
    """Any features defined in the specific edition."""

    uninterpreted_option: List["UninterpretedOption"] = aristaproto.message_field(999)
    """The parser stores options it doesn't recognize here. See above."""

    def __post_init__(self) -> None:
        super().__post_init__()
        if self.is_set("deprecated_legacy_json_field_conflicts"):
            warnings.warn(
                "MessageOptions.deprecated_legacy_json_field_conflicts is deprecated",
                DeprecationWarning,
            )


@dataclass(eq=False, repr=False)
class FieldOptions(aristaproto.Message):
    ctype: "FieldOptionsCType" = aristaproto.enum_field(1)
    """
    The ctype option instructs the C++ code generator to use a different
     representation of the field than it normally would.  See the specific
     options below.  This option is only implemented to support use of
     [ctype=CORD] and [ctype=STRING] (the default) on non-repeated fields of
     type "bytes" in the open source release -- sorry, we'll try to include
     other types in a future version!
    """

    packed: bool = aristaproto.bool_field(2)
    """
    The packed option can be enabled for repeated primitive fields to enable
     a more efficient representation on the wire. Rather than repeatedly
     writing the tag and type for each element, the entire array is encoded as
     a single length-delimited blob. In proto3, only explicit setting it to
     false will avoid using packed encoding.  This option is prohibited in
     Editions, but the `repeated_field_encoding` feature can be used to control
     the behavior.
    """

    jstype: "FieldOptionsJsType" = aristaproto.enum_field(6)
    """
    The jstype option determines the JavaScript type used for values of the
     field.  The option is permitted only for 64 bit integral and fixed types
     (int64, uint64, sint64, fixed64, sfixed64).  A field with jstype JS_STRING
     is represented as JavaScript string, which avoids loss of precision that
     can happen when a large value is converted to a floating point JavaScript.
     Specifying JS_NUMBER for the jstype causes the generated JavaScript code to
     use the JavaScript "number" type.  The behavior of the default option
     JS_NORMAL is implementation dependent.

     This option is an enum to permit additional types to be added, e.g.
     goog.math.Integer.
    """

    lazy: bool = aristaproto.bool_field(5)
    """
    Should this field be parsed lazily?  Lazy applies only to message-type
     fields.  It means that when the outer message is initially parsed, the
     inner message's contents will not be parsed but instead stored in encoded
     form.  The inner message will actually be parsed when it is first accessed.

     This is only a hint.  Implementations are free to choose whether to use
     eager or lazy parsing regardless of the value of this option.  However,
     setting this option true suggests that the protocol author believes that
     using lazy parsing on this field is worth the additional bookkeeping
     overhead typically needed to implement it.

     This option does not affect the public interface of any generated code;
     all method signatures remain the same.  Furthermore, thread-safety of the
     interface is not affected by this option; const methods remain safe to
     call from multiple threads concurrently, while non-const methods continue
     to require exclusive access.

     Note that lazy message fields are still eagerly verified to check
     ill-formed wireformat or missing required fields. Calling IsInitialized()
     on the outer message would fail if the inner message has missing required
     fields. Failed verification would result in parsing failure (except when
     uninitialized messages are acceptable).
    """

    unverified_lazy: bool = aristaproto.bool_field(15)
    """
    unverified_lazy does no correctness checks on the byte stream. This should
     only be used where lazy with verification is prohibitive for performance
     reasons.
    """

    deprecated: bool = aristaproto.bool_field(3)
    """
    Is this field deprecated?
     Depending on the target platform, this can emit Deprecated annotations
     for accessors, or it will be completely ignored; in the very least, this
     is a formalization for deprecating fields.
    """

    weak: bool = aristaproto.bool_field(10)
    """For Google-internal migration only. Do not use."""

    debug_redact: bool = aristaproto.bool_field(16)
    """
    Indicate that the field value should not be printed out when using debug
     formats, e.g. when the field contains sensitive credentials.
    """

    retention: "FieldOptionsOptionRetention" = aristaproto.enum_field(17)
    targets: List["FieldOptionsOptionTargetType"] = aristaproto.enum_field(19)
    edition_defaults: List["FieldOptionsEditionDefault"] = aristaproto.message_field(20)
    features: "FeatureSet" = aristaproto.message_field(21)
    """Any features defined in the specific edition."""

    feature_support: "FieldOptionsFeatureSupport" = aristaproto.message_field(22)
    uninterpreted_option: List["UninterpretedOption"] = aristaproto.message_field(999)
    """The parser stores options it doesn't recognize here. See above."""


@dataclass(eq=False, repr=False)
class FieldOptionsEditionDefault(aristaproto.Message):
    edition: "Edition" = aristaproto.enum_field(3)
    value: str = aristaproto.string_field(2)


@dataclass(eq=False, repr=False)
class FieldOptionsFeatureSupport(aristaproto.Message):
    """Information about the support window of a feature."""

    edition_introduced: "Edition" = aristaproto.enum_field(1)
    """
    The edition that this feature was first available in.  In editions
     earlier than this one, the default assigned to EDITION_LEGACY will be
     used, and proto files will not be able to override it.
    """

    edition_deprecated: "Edition" = aristaproto.enum_field(2)
    """
    The edition this feature becomes deprecated in.  Using this after this
     edition may trigger warnings.
    """

    deprecation_warning: str = aristaproto.string_field(3)
    """
    The deprecation warning text if this feature is used after the edition it
     was marked deprecated in.
    """

    edition_removed: "Edition" = aristaproto.enum_field(4)
    """
    The edition this feature is no longer available in.  In editions after
     this one, the last default assigned will be used, and proto files will
     not be able to override it.
    """


@dataclass(eq=False, repr=False)
class OneofOptions(aristaproto.Message):
    features: "FeatureSet" = aristaproto.message_field(1)
    """Any features defined in the specific edition."""

    uninterpreted_option: List["UninterpretedOption"] = aristaproto.message_field(999)
    """The parser stores options it doesn't recognize here. See above."""


@dataclass(eq=False, repr=False)
class EnumOptions(aristaproto.Message):
    allow_alias: bool = aristaproto.bool_field(2)
    """
    Set this option to true to allow mapping different tag names to the same
     value.
    """

    deprecated: bool = aristaproto.bool_field(3)
    """
    Is this enum deprecated?
     Depending on the target platform, this can emit Deprecated annotations
     for the enum, or it will be completely ignored; in the very least, this
     is a formalization for deprecating enums.
    """

    deprecated_legacy_json_field_conflicts: bool = aristaproto.bool_field(6)
    """
    Enable the legacy handling of JSON field name conflicts.  This lowercases
     and strips underscored from the fields before comparison in proto3 only.
     The new behavior takes `json_name` into account and applies to proto2 as
     well.
     TODO Remove this legacy behavior once downstream teams have
     had time to migrate.
    """

    features: "FeatureSet" = aristaproto.message_field(7)
    """Any features defined in the specific edition."""

    uninterpreted_option: List["UninterpretedOption"] = aristaproto.message_field(999)
    """The parser stores options it doesn't recognize here. See above."""

    def __post_init__(self) -> None:
        super().__post_init__()
        if self.is_set("deprecated_legacy_json_field_conflicts"):
            warnings.warn(
                "EnumOptions.deprecated_legacy_json_field_conflicts is deprecated",
                DeprecationWarning,
            )


@dataclass(eq=False, repr=False)
class EnumValueOptions(aristaproto.Message):
    deprecated: bool = aristaproto.bool_field(1)
    """
    Is this enum value deprecated?
     Depending on the target platform, this can emit Deprecated annotations
     for the enum value, or it will be completely ignored; in the very least,
     this is a formalization for deprecating enum values.
    """

    features: "FeatureSet" = aristaproto.message_field(2)
    """Any features defined in the specific edition."""

    debug_redact: bool = aristaproto.bool_field(3)
    """
    Indicate that fields annotated with this enum value should not be printed
     out when using debug formats, e.g. when the field contains sensitive
     credentials.
    """

    uninterpreted_option: List["UninterpretedOption"] = aristaproto.message_field(999)
    """The parser stores options it doesn't recognize here. See above."""


@dataclass(eq=False, repr=False)
class ServiceOptions(aristaproto.Message):
    features: "FeatureSet" = aristaproto.message_field(34)
    """Any features defined in the specific edition."""

    deprecated: bool = aristaproto.bool_field(33)
    """
    Is this service deprecated?
     Depending on the target platform, this can emit Deprecated annotations
     for the service, or it will be completely ignored; in the very least,
     this is a formalization for deprecating services.
    """

    uninterpreted_option: List["UninterpretedOption"] = aristaproto.message_field(999)
    """The parser stores options it doesn't recognize here. See above."""


@dataclass(eq=False, repr=False)
class MethodOptions(aristaproto.Message):
    deprecated: bool = aristaproto.bool_field(33)
    """
    Is this method deprecated?
     Depending on the target platform, this can emit Deprecated annotations
     for the method, or it will be completely ignored; in the very least,
     this is a formalization for deprecating methods.
    """

    idempotency_level: "MethodOptionsIdempotencyLevel" = aristaproto.enum_field(34)
    features: "FeatureSet" = aristaproto.message_field(35)
    """Any features defined in the specific edition."""

    uninterpreted_option: List["UninterpretedOption"] = aristaproto.message_field(999)
    """The parser stores options it doesn't recognize here. See above."""


@dataclass(eq=False, repr=False)
class UninterpretedOption(aristaproto.Message):
    """
    A message representing a option the parser does not recognize. This only
     appears in options protos created by the compiler::Parser class.
     DescriptorPool resolves these when building Descriptor objects. Therefore,
     options protos in descriptor objects (e.g. returned by Descriptor::options(),
     or produced by Descriptor::CopyTo()) will never have UninterpretedOptions
     in them.
    """

    name: List["UninterpretedOptionNamePart"] = aristaproto.message_field(2)
    identifier_value: str = aristaproto.string_field(3)
    """
    The value of the uninterpreted option, in whatever type the tokenizer
     identified it as during parsing. Exactly one of these should be set.
    """

    positive_int_value: int = aristaproto.uint64_field(4)
    negative_int_value: int = aristaproto.int64_field(5)
    double_value: float = aristaproto.double_field(6)
    string_value: bytes = aristaproto.bytes_field(7)
    aggregate_value: str = aristaproto.string_field(8)


@dataclass(eq=False, repr=False)
class UninterpretedOptionNamePart(aristaproto.Message):
    """
    The name of the uninterpreted option.  Each string represents a segment in
     a dot-separated name.  is_extension is true iff a segment represents an
     extension (denoted with parentheses in options specs in .proto files).
     E.g.,{ ["foo", false], ["bar.baz", true], ["moo", false] } represents
     "foo.(bar.baz).moo".
    """

    name_part: str = aristaproto.string_field(1)
    is_extension: bool = aristaproto.bool_field(2)


@dataclass(eq=False, repr=False)
class FeatureSet(aristaproto.Message):
    """
    TODO Enums in C++ gencode (and potentially other languages) are
     not well scoped.  This means that each of the feature enums below can clash
     with each other.  The short names we've chosen maximize call-site
     readability, but leave us very open to this scenario.  A future feature will
     be designed and implemented to handle this, hopefully before we ever hit a
     conflict here.
    """

    field_presence: "FeatureSetFieldPresence" = aristaproto.enum_field(1)
    enum_type: "FeatureSetEnumType" = aristaproto.enum_field(2)
    repeated_field_encoding: "FeatureSetRepeatedFieldEncoding" = aristaproto.enum_field(
        3
    )
    utf8_validation: "FeatureSetUtf8Validation" = aristaproto.enum_field(4)
    message_encoding: "FeatureSetMessageEncoding" = aristaproto.enum_field(5)
    json_format: "FeatureSetJsonFormat" = aristaproto.enum_field(6)


@dataclass(eq=False, repr=False)
class FeatureSetDefaults(aristaproto.Message):
    """
    A compiled specification for the defaults of a set of features.  These
     messages are generated from FeatureSet extensions and can be used to seed
     feature resolution. The resolution with this object becomes a simple search
     for the closest matching edition, followed by proto merges.
    """

    defaults: List[
        "FeatureSetDefaultsFeatureSetEditionDefault"
    ] = aristaproto.message_field(1)
    minimum_edition: "Edition" = aristaproto.enum_field(4)
    """
    The minimum supported edition (inclusive) when this was constructed.
     Editions before this will not have defaults.
    """

    maximum_edition: "Edition" = aristaproto.enum_field(5)
    """
    The maximum known edition (inclusive) when this was constructed. Editions
     after this will not have reliable defaults.
    """


@dataclass(eq=False, repr=False)
class FeatureSetDefaultsFeatureSetEditionDefault(aristaproto.Message):
    """
    A map from every known edition with a unique set of defaults to its
     defaults. Not all editions may be contained here.  For a given edition,
     the defaults at the closest matching edition ordered at or before it should
     be used.  This field must be in strict ascending order by edition.
    """

    edition: "Edition" = aristaproto.enum_field(3)
    overridable_features: "FeatureSet" = aristaproto.message_field(4)
    """Defaults of features that can be overridden in this edition."""

    fixed_features: "FeatureSet" = aristaproto.message_field(5)
    """Defaults of features that can't be overridden in this edition."""

    features: "FeatureSet" = aristaproto.message_field(2)
    """
    TODO Deprecate and remove this field, which is just the
     above two merged.
    """


@dataclass(eq=False, repr=False)
class SourceCodeInfo(aristaproto.Message):
    """
    Encapsulates information about the original source file from which a
     FileDescriptorProto was generated.
    """

    location: List["SourceCodeInfoLocation"] = aristaproto.message_field(1)
    """
    A Location identifies a piece of source code in a .proto file which
     corresponds to a particular definition.  This information is intended
     to be useful to IDEs, code indexers, documentation generators, and similar
     tools.

     For example, say we have a file like:
       message Foo {
         optional string foo = 1;
       }
     Let's look at just the field definition:
       optional string foo = 1;
       ^       ^^     ^^  ^  ^^^
       a       bc     de  f  ghi
     We have the following locations:
       span   path               represents
       [a,i)  [ 4, 0, 2, 0 ]     The whole field definition.
       [a,b)  [ 4, 0, 2, 0, 4 ]  The label (optional).
       [c,d)  [ 4, 0, 2, 0, 5 ]  The type (string).
       [e,f)  [ 4, 0, 2, 0, 1 ]  The name (foo).
       [g,h)  [ 4, 0, 2, 0, 3 ]  The number (1).

     Notes:
     - A location may refer to a repeated field itself (i.e. not to any
       particular index within it).  This is used whenever a set of elements are
       logically enclosed in a single code segment.  For example, an entire
       extend block (possibly containing multiple extension definitions) will
       have an outer location whose path refers to the "extensions" repeated
       field without an index.
     - Multiple locations may have the same path.  This happens when a single
       logical declaration is spread out across multiple places.  The most
       obvious example is the "extend" block again -- there may be multiple
       extend blocks in the same scope, each of which will have the same path.
     - A location's span is not always a subset of its parent's span.  For
       example, the "extendee" of an extension declaration appears at the
       beginning of the "extend" block and is shared by all extensions within
       the block.
     - Just because a location's span is a subset of some other location's span
       does not mean that it is a descendant.  For example, a "group" defines
       both a type and a field in a single declaration.  Thus, the locations
       corresponding to the type and field and their components will overlap.
     - Code which tries to interpret locations should probably be designed to
       ignore those that it doesn't understand, as more types of locations could
       be recorded in the future.
    """


@dataclass(eq=False, repr=False)
class SourceCodeInfoLocation(aristaproto.Message):
    path: List[int] = aristaproto.int32_field(1)
    """
    Identifies which part of the FileDescriptorProto was defined at this
     location.

     Each element is a field number or an index.  They form a path from
     the root FileDescriptorProto to the place where the definition appears.
     For example, this path:
       [ 4, 3, 2, 7, 1 ]
     refers to:
       file.message_type(3)  // 4, 3
           .field(7)         // 2, 7
           .name()           // 1
     This is because FileDescriptorProto.message_type has field number 4:
       repeated DescriptorProto message_type = 4;
     and DescriptorProto.field has field number 2:
       repeated FieldDescriptorProto field = 2;
     and FieldDescriptorProto.name has field number 1:
       optional string name = 1;

     Thus, the above path gives the location of a field name.  If we removed
     the last element:
       [ 4, 3, 2, 7 ]
     this path refers to the whole field declaration (from the beginning
     of the label to the terminating semicolon).
    """

    span: List[int] = aristaproto.int32_field(2)
    """
    Always has exactly three or four elements: start line, start column,
     end line (optional, otherwise assumed same as start line), end column.
     These are packed into a single field for efficiency.  Note that line
     and column numbers are zero-based -- typically you will want to add
     1 to each before displaying to a user.
    """

    leading_comments: str = aristaproto.string_field(3)
    """
    If this SourceCodeInfo represents a complete declaration, these are any
     comments appearing before and after the declaration which appear to be
     attached to the declaration.

     A series of line comments appearing on consecutive lines, with no other
     tokens appearing on those lines, will be treated as a single comment.

     leading_detached_comments will keep paragraphs of comments that appear
     before (but not connected to) the current element. Each paragraph,
     separated by empty lines, will be one comment element in the repeated
     field.

     Only the comment content is provided; comment markers (e.g. //) are
     stripped out.  For block comments, leading whitespace and an asterisk
     will be stripped from the beginning of each line other than the first.
     Newlines are included in the output.

     Examples:

       optional int32 foo = 1;  // Comment attached to foo.
       // Comment attached to bar.
       optional int32 bar = 2;

       optional string baz = 3;
       // Comment attached to baz.
       // Another line attached to baz.

       // Comment attached to moo.
       //
       // Another line attached to moo.
       optional double moo = 4;

       // Detached comment for corge. This is not leading or trailing comments
       // to moo or corge because there are blank lines separating it from
       // both.

       // Detached comment for corge paragraph 2.

       optional string corge = 5;
       /* Block comment attached
        * to corge.  Leading asterisks
        * will be removed. */
       /* Block comment attached to
        * grault. */
       optional int32 grault = 6;

       // ignored detached comments.
    """

    trailing_comments: str = aristaproto.string_field(4)
    leading_detached_comments: List[str] = aristaproto.string_field(6)


@dataclass(eq=False, repr=False)
class GeneratedCodeInfo(aristaproto.Message):
    """
    Describes the relationship between generated code and its original source
     file. A GeneratedCodeInfo message is associated with only one generated
     source file, but may contain references to different source .proto files.
    """

    annotation: List["GeneratedCodeInfoAnnotation"] = aristaproto.message_field(1)
    """
    An Annotation connects some span of text in generated code to an element
     of its generating .proto file.
    """


@dataclass(eq=False, repr=False)
class GeneratedCodeInfoAnnotation(aristaproto.Message):
    path: List[int] = aristaproto.int32_field(1)
    """
    Identifies the element in the original source .proto file. This field
     is formatted the same as SourceCodeInfo.Location.path.
    """

    source_file: str = aristaproto.string_field(2)
    """Identifies the filesystem path to the original source .proto."""

    begin: int = aristaproto.int32_field(3)
    """
    Identifies the starting offset in bytes in the generated code
     that relates to the identified object.
    """

    end: int = aristaproto.int32_field(4)
    """
    Identifies the ending offset in bytes in the generated code that
     relates to the identified object. The end offset should be one past
     the last relevant byte (so the length of the text = end - begin).
    """

    semantic: "GeneratedCodeInfoAnnotationSemantic" = aristaproto.enum_field(5)


@dataclass(eq=False, repr=False)
class Duration(aristaproto.Message):
    """
    A Duration represents a signed, fixed-length span of time represented
     as a count of seconds and fractions of seconds at nanosecond
     resolution. It is independent of any calendar and concepts like "day"
     or "month". It is related to Timestamp in that the difference between
     two Timestamp values is a Duration and it can be added or subtracted
     from a Timestamp. Range is approximately +-10,000 years.

     # Examples

     Example 1: Compute Duration from two Timestamps in pseudo code.

         Timestamp start = ...;
         Timestamp end = ...;
         Duration duration = ...;

         duration.seconds = end.seconds - start.seconds;
         duration.nanos = end.nanos - start.nanos;

         if (duration.seconds < 0 && duration.nanos > 0) {
           duration.seconds += 1;
           duration.nanos -= 1000000000;
         } else if (duration.seconds > 0 && duration.nanos < 0) {
           duration.seconds -= 1;
           duration.nanos += 1000000000;
         }

     Example 2: Compute Timestamp from Timestamp + Duration in pseudo code.

         Timestamp start = ...;
         Duration duration = ...;
         Timestamp end = ...;

         end.seconds = start.seconds + duration.seconds;
         end.nanos = start.nanos + duration.nanos;

         if (end.nanos < 0) {
           end.seconds -= 1;
           end.nanos += 1000000000;
         } else if (end.nanos >= 1000000000) {
           end.seconds += 1;
           end.nanos -= 1000000000;
         }

     Example 3: Compute Duration from datetime.timedelta in Python.

         td = datetime.timedelta(days=3, minutes=10)
         duration = Duration()
         duration.FromTimedelta(td)

     # JSON Mapping

     In JSON format, the Duration type is encoded as a string rather than an
     object, where the string ends in the suffix "s" (indicating seconds) and
     is preceded by the number of seconds, with nanoseconds expressed as
     fractional seconds. For example, 3 seconds with 0 nanoseconds should be
     encoded in JSON format as "3s", while 3 seconds and 1 nanosecond should
     be expressed in JSON format as "3.000000001s", and 3 seconds and 1
     microsecond should be expressed in JSON format as "3.000001s".
    """

    seconds: int = aristaproto.int64_field(1)
    """
    Signed seconds of the span of time. Must be from -315,576,000,000
     to +315,576,000,000 inclusive. Note: these bounds are computed from:
     60 sec/min * 60 min/hr * 24 hr/day * 365.25 days/year * 10000 years
    """

    nanos: int = aristaproto.int32_field(2)
    """
    Signed fractions of a second at nanosecond resolution of the span
     of time. Durations less than one second are represented with a 0
     `seconds` field and a positive or negative `nanos` field. For durations
     of one second or more, a non-zero value for the `nanos` field must be
     of the same sign as the `seconds` field. Must be from -999,999,999
     to +999,999,999 inclusive.
    """


@dataclass(eq=False, repr=False)
class Empty(aristaproto.Message):
    """
    A generic empty message that you can re-use to avoid defining duplicated
     empty messages in your APIs. A typical example is to use it as the request
     or the response type of an API method. For instance:

         service Foo {
           rpc Bar(google.protobuf.Empty) returns (google.protobuf.Empty);
         }
    """

    pass


@dataclass(eq=False, repr=False)
class FieldMask(aristaproto.Message):
    """
    `FieldMask` represents a set of symbolic field paths, for example:

         paths: "f.a"
         paths: "f.b.d"

     Here `f` represents a field in some root message, `a` and `b`
     fields in the message found in `f`, and `d` a field found in the
     message in `f.b`.

     Field masks are used to specify a subset of fields that should be
     returned by a get operation or modified by an update operation.
     Field masks also have a custom JSON encoding (see below).

     # Field Masks in Projections

     When used in the context of a projection, a response message or
     sub-message is filtered by the API to only contain those fields as
     specified in the mask. For example, if the mask in the previous
     example is applied to a response message as follows:

         f {
           a : 22
           b {
             d : 1
             x : 2
           }
           y : 13
         }
         z: 8

     The result will not contain specific values for fields x,y and z
     (their value will be set to the default, and omitted in proto text
     output):


         f {
           a : 22
           b {
             d : 1
           }
         }

     A repeated field is not allowed except at the last position of a
     paths string.

     If a FieldMask object is not present in a get operation, the
     operation applies to all fields (as if a FieldMask of all fields
     had been specified).

     Note that a field mask does not necessarily apply to the
     top-level response message. In case of a REST get operation, the
     field mask applies directly to the response, but in case of a REST
     list operation, the mask instead applies to each individual message
     in the returned resource list. In case of a REST custom method,
     other definitions may be used. Where the mask applies will be
     clearly documented together with its declaration in the API.  In
     any case, the effect on the returned resource/resources is required
     behavior for APIs.

     # Field Masks in Update Operations

     A field mask in update operations specifies which fields of the
     targeted resource are going to be updated. The API is required
     to only change the values of the fields as specified in the mask
     and leave the others untouched. If a resource is passed in to
     describe the updated values, the API ignores the values of all
     fields not covered by the mask.

     If a repeated field is specified for an update operation, new values will
     be appended to the existing repeated field in the target resource. Note that
     a repeated field is only allowed in the last position of a `paths` string.

     If a sub-message is specified in the last position of the field mask for an
     update operation, then new value will be merged into the existing sub-message
     in the target resource.

     For example, given the target message:

         f {
           b {
             d: 1
             x: 2
           }
           c: [1]
         }

     And an update message:

         f {
           b {
             d: 10
           }
           c: [2]
         }

     then if the field mask is:

      paths: ["f.b", "f.c"]

     then the result will be:

         f {
           b {
             d: 10
             x: 2
           }
           c: [1, 2]
         }

     An implementation may provide options to override this default behavior for
     repeated and message fields.

     In order to reset a field's value to the default, the field must
     be in the mask and set to the default value in the provided resource.
     Hence, in order to reset all fields of a resource, provide a default
     instance of the resource and set all fields in the mask, or do
     not provide a mask as described below.

     If a field mask is not present on update, the operation applies to
     all fields (as if a field mask of all fields has been specified).
     Note that in the presence of schema evolution, this may mean that
     fields the client does not know and has therefore not filled into
     the request will be reset to their default. If this is unwanted
     behavior, a specific service may require a client to always specify
     a field mask, producing an error if not.

     As with get operations, the location of the resource which
     describes the updated values in the request message depends on the
     operation kind. In any case, the effect of the field mask is
     required to be honored by the API.

     ## Considerations for HTTP REST

     The HTTP kind of an update operation which uses a field mask must
     be set to PATCH instead of PUT in order to satisfy HTTP semantics
     (PUT must only be used for full updates).

     # JSON Encoding of Field Masks

     In JSON, a field mask is encoded as a single string where paths are
     separated by a comma. Fields name in each path are converted
     to/from lower-camel naming conventions.

     As an example, consider the following message declarations:

         message Profile {
           User user = 1;
           Photo photo = 2;
         }
         message User {
           string display_name = 1;
           string address = 2;
         }

     In proto a field mask for `Profile` may look as such:

         mask {
           paths: "user.display_name"
           paths: "photo"
         }

     In JSON, the same mask is represented as below:

         {
           mask: "user.displayName,photo"
         }

     # Field Masks and Oneof Fields

     Field masks treat fields in oneofs just as regular fields. Consider the
     following message:

         message SampleMessage {
           oneof test_oneof {
             string name = 4;
             SubMessage sub_message = 9;
           }
         }

     The field mask can be:

         mask {
           paths: "name"
         }

     Or:

         mask {
           paths: "sub_message"
         }

     Note that oneof type names ("test_oneof" in this case) cannot be used in
     paths.

     ## Field Mask Verification

     The implementation of any API method which has a FieldMask type field in the
     request should verify the included field paths, and return an
     `INVALID_ARGUMENT` error if any path is unmappable.
    """

    paths: List[str] = aristaproto.string_field(1)
    """The set of field mask paths."""


@dataclass(eq=False, repr=False)
class Struct(aristaproto.Message):
    """
    `Struct` represents a structured data value, consisting of fields
     which map to dynamically typed values. In some languages, `Struct`
     might be supported by a native representation. For example, in
     scripting languages like JS a struct is represented as an
     object. The details of that representation are described together
     with the proto support for the language.

     The JSON representation for `Struct` is JSON object.
    """

    fields: Dict[str, "Value"] = aristaproto.map_field(
        1, aristaproto.TYPE_STRING, aristaproto.TYPE_MESSAGE
    )
    """Unordered map of dynamically typed values."""

    @hybridmethod
    def from_dict(cls: "type[Self]", value: Mapping[str, Any]) -> Self:  # type: ignore
        self = cls()
        return self.from_dict(value)

    @from_dict.instancemethod
    def from_dict(self, value: Mapping[str, Any]) -> Self:
        fields = {**value}
        for k in fields:
            if hasattr(fields[k], "from_dict"):
                fields[k] = fields[k].from_dict()

        self.fields = fields
        return self

    def to_dict(
        self,
        casing: aristaproto.Casing = aristaproto.Casing.CAMEL,
        include_default_values: bool = False,
    ) -> Dict[str, Any]:
        output = {**self.fields}
        for k in self.fields:
            if hasattr(self.fields[k], "to_dict"):
                output[k] = self.fields[k].to_dict(casing, include_default_values)
        return output


@dataclass(eq=False, repr=False)
class Value(aristaproto.Message):
    """
    `Value` represents a dynamically typed value which can be either
     null, a number, a string, a boolean, a recursive struct value, or a
     list of values. A producer of value is expected to set one of these
     variants. Absence of any variant indicates an error.

     The JSON representation for `Value` is JSON value.
    """

    null_value: Optional["NullValue"] = aristaproto.enum_field(
        1, optional=True, group="kind"
    )
    """Represents a null value."""

    number_value: Optional[float] = aristaproto.double_field(
        2, optional=True, group="kind"
    )
    """Represents a double value."""

    string_value: Optional[str] = aristaproto.string_field(
        3, optional=True, group="kind"
    )
    """Represents a string value."""

    bool_value: Optional[bool] = aristaproto.bool_field(4, optional=True, group="kind")
    """Represents a boolean value."""

    struct_value: Optional["Struct"] = aristaproto.message_field(
        5, optional=True, group="kind"
    )
    """Represents a structured value."""

    list_value: Optional["ListValue"] = aristaproto.message_field(
        6, optional=True, group="kind"
    )
    """Represents a repeated `Value`."""

    @root_validator()
    def check_oneof(cls, values):
        return cls._validate_field_groups(values)


@dataclass(eq=False, repr=False)
class ListValue(aristaproto.Message):
    """
    `ListValue` is a wrapper around a repeated field of values.

     The JSON representation for `ListValue` is JSON array.
    """

    values: List["Value"] = aristaproto.message_field(1)
    """Repeated field of dynamically typed values."""


@dataclass(eq=False, repr=False)
class Timestamp(aristaproto.Message):
    """
    A Timestamp represents a point in time independent of any time zone or local
     calendar, encoded as a count of seconds and fractions of seconds at
     nanosecond resolution. The count is relative to an epoch at UTC midnight on
     January 1, 1970, in the proleptic Gregorian calendar which extends the
     Gregorian calendar backwards to year one.

     All minutes are 60 seconds long. Leap seconds are "smeared" so that no leap
     second table is needed for interpretation, using a [24-hour linear
     smear](https://developers.google.com/time/smear).

     The range is from 0001-01-01T00:00:00Z to 9999-12-31T23:59:59.999999999Z. By
     restricting to that range, we ensure that we can convert to and from [RFC
     3339](https://www.ietf.org/rfc/rfc3339.txt) date strings.

     # Examples

     Example 1: Compute Timestamp from POSIX `time()`.

         Timestamp timestamp;
         timestamp.set_seconds(time(NULL));
         timestamp.set_nanos(0);

     Example 2: Compute Timestamp from POSIX `gettimeofday()`.

         struct timeval tv;
         gettimeofday(&tv, NULL);

         Timestamp timestamp;
         timestamp.set_seconds(tv.tv_sec);
         timestamp.set_nanos(tv.tv_usec * 1000);

     Example 3: Compute Timestamp from Win32 `GetSystemTimeAsFileTime()`.

         FILETIME ft;
         GetSystemTimeAsFileTime(&ft);
         UINT64 ticks = (((UINT64)ft.dwHighDateTime) << 32) | ft.dwLowDateTime;

         // A Windows tick is 100 nanoseconds. Windows epoch 1601-01-01T00:00:00Z
         // is 11644473600 seconds before Unix epoch 1970-01-01T00:00:00Z.
         Timestamp timestamp;
         timestamp.set_seconds((INT64) ((ticks / 10000000) - 11644473600LL));
         timestamp.set_nanos((INT32) ((ticks % 10000000) * 100));

     Example 4: Compute Timestamp from Java `System.currentTimeMillis()`.

         long millis = System.currentTimeMillis();

         Timestamp timestamp = Timestamp.newBuilder().setSeconds(millis / 1000)
             .setNanos((int) ((millis % 1000) * 1000000)).build();

     Example 5: Compute Timestamp from Java `Instant.now()`.

         Instant now = Instant.now();

         Timestamp timestamp =
             Timestamp.newBuilder().setSeconds(now.getEpochSecond())
                 .setNanos(now.getNano()).build();

     Example 6: Compute Timestamp from current time in Python.

         timestamp = Timestamp()
         timestamp.GetCurrentTime()

     # JSON Mapping

     In JSON format, the Timestamp type is encoded as a string in the
     [RFC 3339](https://www.ietf.org/rfc/rfc3339.txt) format. That is, the
     format is "{year}-{month}-{day}T{hour}:{min}:{sec}[.{frac_sec}]Z"
     where {year} is always expressed using four digits while {month}, {day},
     {hour}, {min}, and {sec} are zero-padded to two digits each. The fractional
     seconds, which can go up to 9 digits (i.e. up to 1 nanosecond resolution),
     are optional. The "Z" suffix indicates the timezone ("UTC"); the timezone
     is required. A proto3 JSON serializer should always use UTC (as indicated by
     "Z") when printing the Timestamp type and a proto3 JSON parser should be
     able to accept both UTC and other timezones (as indicated by an offset).

     For example, "2017-01-15T01:30:15.01Z" encodes 15.01 seconds past
     01:30 UTC on January 15, 2017.

     In JavaScript, one can convert a Date object to this format using the
     standard
     [toISOString()](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString)
     method. In Python, a standard `datetime.datetime` object can be converted
     to this format using
     [`strftime`](https://docs.python.org/2/library/time.html#time.strftime) with
     the time format spec '%Y-%m-%dT%H:%M:%S.%fZ'. Likewise, in Java, one can use
     the Joda Time's [`ISODateTimeFormat.dateTime()`](
     http://joda-time.sourceforge.net/apidocs/org/joda/time/format/ISODateTimeFormat.html#dateTime()
     ) to obtain a formatter capable of generating timestamps in this format.
    """

    seconds: int = aristaproto.int64_field(1)
    """
    Represents seconds of UTC time since Unix epoch
     1970-01-01T00:00:00Z. Must be from 0001-01-01T00:00:00Z to
     9999-12-31T23:59:59Z inclusive.
    """

    nanos: int = aristaproto.int32_field(2)
    """
    Non-negative fractions of a second at nanosecond resolution. Negative
     second values with fractions must still have non-negative nanos values
     that count forward in time. Must be from 0 to 999,999,999
     inclusive.
    """


@dataclass(eq=False, repr=False)
class DoubleValue(aristaproto.Message):
    """
    Wrapper message for `double`.

     The JSON representation for `DoubleValue` is JSON number.
    """

    value: float = aristaproto.double_field(1)
    """The double value."""


@dataclass(eq=False, repr=False)
class FloatValue(aristaproto.Message):
    """
    Wrapper message for `float`.

     The JSON representation for `FloatValue` is JSON number.
    """

    value: float = aristaproto.float_field(1)
    """The float value."""


@dataclass(eq=False, repr=False)
class Int64Value(aristaproto.Message):
    """
    Wrapper message for `int64`.

     The JSON representation for `Int64Value` is JSON string.
    """

    value: int = aristaproto.int64_field(1)
    """The int64 value."""


@dataclass(eq=False, repr=False)
class UInt64Value(aristaproto.Message):
    """
    Wrapper message for `uint64`.

     The JSON representation for `UInt64Value` is JSON string.
    """

    value: int = aristaproto.uint64_field(1)
    """The uint64 value."""


@dataclass(eq=False, repr=False)
class Int32Value(aristaproto.Message):
    """
    Wrapper message for `int32`.

     The JSON representation for `Int32Value` is JSON number.
    """

    value: int = aristaproto.int32_field(1)
    """The int32 value."""


@dataclass(eq=False, repr=False)
class UInt32Value(aristaproto.Message):
    """
    Wrapper message for `uint32`.

     The JSON representation for `UInt32Value` is JSON number.
    """

    value: int = aristaproto.uint32_field(1)
    """The uint32 value."""


@dataclass(eq=False, repr=False)
class BoolValue(aristaproto.Message):
    """
    Wrapper message for `bool`.

     The JSON representation for `BoolValue` is JSON `true` and `false`.
    """

    value: bool = aristaproto.bool_field(1)
    """The bool value."""


@dataclass(eq=False, repr=False)
class StringValue(aristaproto.Message):
    """
    Wrapper message for `string`.

     The JSON representation for `StringValue` is JSON string.
    """

    value: str = aristaproto.string_field(1)
    """The string value."""


@dataclass(eq=False, repr=False)
class BytesValue(aristaproto.Message):
    """
    Wrapper message for `bytes`.

     The JSON representation for `BytesValue` is JSON string.
    """

    value: bytes = aristaproto.bytes_field(1)
    """The bytes value."""


Type.__pydantic_model__.update_forward_refs()  # type: ignore
Field.__pydantic_model__.update_forward_refs()  # type: ignore
Enum.__pydantic_model__.update_forward_refs()  # type: ignore
EnumValue.__pydantic_model__.update_forward_refs()  # type: ignore
Option.__pydantic_model__.update_forward_refs()  # type: ignore
Api.__pydantic_model__.update_forward_refs()  # type: ignore
Method.__pydantic_model__.update_forward_refs()  # type: ignore
FileDescriptorSet.__pydantic_model__.update_forward_refs()  # type: ignore
FileDescriptorProto.__pydantic_model__.update_forward_refs()  # type: ignore
DescriptorProto.__pydantic_model__.update_forward_refs()  # type: ignore
DescriptorProtoExtensionRange.__pydantic_model__.update_forward_refs()  # type: ignore
ExtensionRangeOptions.__pydantic_model__.update_forward_refs()  # type: ignore
FieldDescriptorProto.__pydantic_model__.update_forward_refs()  # type: ignore
OneofDescriptorProto.__pydantic_model__.update_forward_refs()  # type: ignore
EnumDescriptorProto.__pydantic_model__.update_forward_refs()  # type: ignore
EnumValueDescriptorProto.__pydantic_model__.update_forward_refs()  # type: ignore
ServiceDescriptorProto.__pydantic_model__.update_forward_refs()  # type: ignore
MethodDescriptorProto.__pydantic_model__.update_forward_refs()  # type: ignore
FileOptions.__pydantic_model__.update_forward_refs()  # type: ignore
MessageOptions.__pydantic_model__.update_forward_refs()  # type: ignore
FieldOptions.__pydantic_model__.update_forward_refs()  # type: ignore
FieldOptionsEditionDefault.__pydantic_model__.update_forward_refs()  # type: ignore
FieldOptionsFeatureSupport.__pydantic_model__.update_forward_refs()  # type: ignore
OneofOptions.__pydantic_model__.update_forward_refs()  # type: ignore
EnumOptions.__pydantic_model__.update_forward_refs()  # type: ignore
EnumValueOptions.__pydantic_model__.update_forward_refs()  # type: ignore
ServiceOptions.__pydantic_model__.update_forward_refs()  # type: ignore
MethodOptions.__pydantic_model__.update_forward_refs()  # type: ignore
UninterpretedOption.__pydantic_model__.update_forward_refs()  # type: ignore
FeatureSet.__pydantic_model__.update_forward_refs()  # type: ignore
FeatureSetDefaults.__pydantic_model__.update_forward_refs()  # type: ignore
FeatureSetDefaultsFeatureSetEditionDefault.__pydantic_model__.update_forward_refs()  # type: ignore
SourceCodeInfo.__pydantic_model__.update_forward_refs()  # type: ignore
GeneratedCodeInfo.__pydantic_model__.update_forward_refs()  # type: ignore
GeneratedCodeInfoAnnotation.__pydantic_model__.update_forward_refs()  # type: ignore
Struct.__pydantic_model__.update_forward_refs()  # type: ignore
Value.__pydantic_model__.update_forward_refs()  # type: ignore
ListValue.__pydantic_model__.update_forward_refs()  # type: ignore



from tests.output_aristaproto.casing_message_field_uppercase import Test


def test_message_casing():
    message = Test()
    assert hasattr(
        message, "uppercase"
    ), "UPPERCASE attribute is converted to 'uppercase' in python"
    assert hasattr(
        message, "uppercase_v2"
    ), "UPPERCASE_V2 attribute is converted to 'uppercase_v2' in python"
    assert hasattr(
        message, "upper_camel_case"
    ), "UPPER_CAMEL_CASE attribute is converted to upper_camel_case in python"



import datetime

import pytest

import aristaproto
from tests.output_aristaproto.oneof_default_value_serialization import (
    Message,
    NestedMessage,
    Test,
)


def assert_round_trip_serialization_works(message: Test) -> None:
    assert aristaproto.which_one_of(message, "value_type") == aristaproto.which_one_of(
        Test().from_json(message.to_json()), "value_type"
    )


def test_oneof_default_value_serialization_works_for_all_values():
    """
    Serialization from message with oneof set to default -> JSON -> message should keep
    default value field intact.
    """

    test_cases = [
        Test(bool_value=False),
        Test(int64_value=0),
        Test(
            timestamp_value=datetime.datetime(
                year=1970,
                month=1,
                day=1,
                hour=0,
                minute=0,
                tzinfo=datetime.timezone.utc,
            )
        ),
        Test(duration_value=datetime.timedelta(0)),
        Test(wrapped_message_value=Message(value=0)),
        # NOTE: Do NOT use aristaproto.BoolValue here, it will cause JSON serialization
        # errors.
        # TODO: Do we want to allow use of BoolValue directly within a wrapped field or
        # should we simply hard fail here?
        Test(wrapped_bool_value=False),
    ]
    for message in test_cases:
        assert_round_trip_serialization_works(message)


def test_oneof_no_default_values_passed():
    message = Test()
    assert (
        aristaproto.which_one_of(message, "value_type")
        == aristaproto.which_one_of(Test().from_json(message.to_json()), "value_type")
        == ("", None)
    )


def test_oneof_nested_oneof_messages_are_serialized_with_defaults():
    """
    Nested messages with oneofs should also be handled
    """
    message = Test(
        wrapped_nested_message_value=NestedMessage(
            id=0, wrapped_message_value=Message(value=0)
        )
    )
    assert (
        aristaproto.which_one_of(message, "value_type")
        == aristaproto.which_one_of(Test().from_json(message.to_json()), "value_type")
        == (
            "wrapped_nested_message_value",
            NestedMessage(id=0, wrapped_message_value=Message(value=0)),
        )
    )



from tests.output_aristaproto.enum import (
    ArithmeticOperator,
    Choice,
    Test,
)


def test_enum_set_and_get():
    assert Test(choice=Choice.ZERO).choice == Choice.ZERO
    assert Test(choice=Choice.ONE).choice == Choice.ONE
    assert Test(choice=Choice.THREE).choice == Choice.THREE
    assert Test(choice=Choice.FOUR).choice == Choice.FOUR


def test_enum_set_with_int():
    assert Test(choice=0).choice == Choice.ZERO
    assert Test(choice=1).choice == Choice.ONE
    assert Test(choice=3).choice == Choice.THREE
    assert Test(choice=4).choice == Choice.FOUR


def test_enum_is_comparable_with_int():
    assert Test(choice=Choice.ZERO).choice == 0
    assert Test(choice=Choice.ONE).choice == 1
    assert Test(choice=Choice.THREE).choice == 3
    assert Test(choice=Choice.FOUR).choice == 4


def test_enum_to_dict():
    assert (
        "choice" not in Test(choice=Choice.ZERO).to_dict()
    ), "Default enum value is not serialized"
    assert (
        Test(choice=Choice.ZERO).to_dict(include_default_values=True)["choice"]
        == "ZERO"
    )
    assert Test(choice=Choice.ONE).to_dict()["choice"] == "ONE"
    assert Test(choice=Choice.THREE).to_dict()["choice"] == "THREE"
    assert Test(choice=Choice.FOUR).to_dict()["choice"] == "FOUR"


def test_repeated_enum_is_comparable_with_int():
    assert Test(choices=[Choice.ZERO]).choices == [0]
    assert Test(choices=[Choice.ONE]).choices == [1]
    assert Test(choices=[Choice.THREE]).choices == [3]
    assert Test(choices=[Choice.FOUR]).choices == [4]


def test_repeated_enum_set_and_get():
    assert Test(choices=[Choice.ZERO]).choices == [Choice.ZERO]
    assert Test(choices=[Choice.ONE]).choices == [Choice.ONE]
    assert Test(choices=[Choice.THREE]).choices == [Choice.THREE]
    assert Test(choices=[Choice.FOUR]).choices == [Choice.FOUR]


def test_repeated_enum_to_dict():
    assert Test(choices=[Choice.ZERO]).to_dict()["choices"] == ["ZERO"]
    assert Test(choices=[Choice.ONE]).to_dict()["choices"] == ["ONE"]
    assert Test(choices=[Choice.THREE]).to_dict()["choices"] == ["THREE"]
    assert Test(choices=[Choice.FOUR]).to_dict()["choices"] == ["FOUR"]

    all_enums_dict = Test(
        choices=[Choice.ZERO, Choice.ONE, Choice.THREE, Choice.FOUR]
    ).to_dict()
    assert (all_enums_dict["choices"]) == ["ZERO", "ONE", "THREE", "FOUR"]


def test_repeated_enum_with_single_value_to_dict():
    assert Test(choices=Choice.ONE).to_dict()["choices"] == ["ONE"]
    assert Test(choices=1).to_dict()["choices"] == ["ONE"]


def test_repeated_enum_with_non_list_iterables_to_dict():
    assert Test(choices=(1, 3)).to_dict()["choices"] == ["ONE", "THREE"]
    assert Test(choices=(1, 3)).to_dict()["choices"] == ["ONE", "THREE"]
    assert Test(choices=(Choice.ONE, Choice.THREE)).to_dict()["choices"] == [
        "ONE",
        "THREE",
    ]

    def enum_generator():
        yield Choice.ONE
        yield Choice.THREE

    assert Test(choices=enum_generator()).to_dict()["choices"] == ["ONE", "THREE"]


def test_enum_mapped_on_parse():
    # test default value
    b = Test().parse(bytes(Test()))
    assert b.choice.name == Choice.ZERO.name
    assert b.choices == []

    # test non default value
    a = Test().parse(bytes(Test(choice=Choice.ONE)))
    assert a.choice.name == Choice.ONE.name
    assert b.choices == []

    # test repeated
    c = Test().parse(bytes(Test(choices=[Choice.THREE, Choice.FOUR])))
    assert c.choices[0].name == Choice.THREE.name
    assert c.choices[1].name == Choice.FOUR.name

    # bonus: defaults after empty init are also mapped
    assert Test().choice.name == Choice.ZERO.name


def test_renamed_enum_members():
    assert set(ArithmeticOperator.__members__) == {
        "NONE",
        "PLUS",
        "MINUS",
        "_0_PREFIXED",
    }



from typing import List

from grpclib.client import Channel


class MockChannel(Channel):
    # noinspection PyMissingConstructor
    def __init__(self, responses=None) -> None:
        self.responses = responses or []
        self.requests = []
        self._loop = None

    def request(self, route, cardinality, request, response_type, **kwargs):
        self.requests.append(
            {
                "route": route,
                "cardinality": cardinality,
                "request": request,
                "response_type": response_type,
            }
        )
        return MockStream(self.responses)


class MockStream:
    def __init__(self, responses: List) -> None:
        super().__init__()
        self.responses = responses

    async def recv_message(self):
        return self.responses.pop(0)

    async def send_message(self, *args, **kwargs):
        pass

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        return True

    async def __aenter__(self):
        return self



import inspect

from tests.output_aristaproto.service_uppercase import TestStub


def test_parameters():
    sig = inspect.signature(TestStub.do_thing)
    assert len(sig.parameters) == 5, "Expected 5 parameters"



import json

from tests.output_aristaproto.proto3_field_presence import (
    InnerTest,
    Test,
    TestEnum,
)


def test_null_fields_json():
    """Ensure that using "null" in JSON is equivalent to not specifying a
    field, for fields with explicit presence"""

    def test_json(ref_json: str, obj_json: str) -> None:
        """`ref_json` and `obj_json` are JSON strings describing a `Test` object.
        Test that deserializing both leads to the same object, and that
        `ref_json` is the normalized format."""
        ref_obj = Test().from_json(ref_json)
        obj = Test().from_json(obj_json)

        assert obj == ref_obj
        assert json.loads(obj.to_json(0)) == json.loads(ref_json)

    test_json("{}", '{ "test1": null, "test2": null, "test3": null }')
    test_json("{}", '{ "test4": null, "test5": null, "test6": null }')
    test_json("{}", '{ "test7": null, "test8": null }')
    test_json('{ "test5": {} }', '{ "test3": null, "test5": {} }')

    # Make sure that if include_default_values is set, None values are
    # exported.
    obj = Test()
    assert obj.to_dict() == {}
    assert obj.to_dict(include_default_values=True) == {
        "test1": None,
        "test2": None,
        "test3": None,
        "test4": None,
        "test5": None,
        "test6": None,
        "test7": None,
        "test8": None,
        "test9": None,
    }


def test_unset_access():  # see #523
    assert Test().test1 is None
    assert Test(test1=None).test1 is None



import asyncio
from dataclasses import dataclass
from typing import AsyncIterator

import pytest

import aristaproto
from aristaproto.grpc.util.async_channel import AsyncChannel


@dataclass
class Message(aristaproto.Message):
    body: str = aristaproto.string_field(1)


@pytest.fixture
def expected_responses():
    return [Message("Hello world 1"), Message("Hello world 2"), Message("Done")]


class ClientStub:
    async def connect(self, requests: AsyncIterator):
        await asyncio.sleep(0.1)
        async for request in requests:
            await asyncio.sleep(0.1)
            yield request
        await asyncio.sleep(0.1)
        yield Message("Done")


async def to_list(generator: AsyncIterator):
    return [value async for value in generator]


@pytest.fixture
def client():
    # channel = Channel(host='127.0.0.1', port=50051)
    # return ClientStub(channel)
    return ClientStub()


@pytest.mark.asyncio
async def test_send_from_before_connect_and_close_automatically(
    client, expected_responses
):
    requests = AsyncChannel()
    await requests.send_from(
        [Message(body="Hello world 1"), Message(body="Hello world 2")], close=True
    )
    responses = client.connect(requests)

    assert await to_list(responses) == expected_responses


@pytest.mark.asyncio
async def test_send_from_after_connect_and_close_automatically(
    client, expected_responses
):
    requests = AsyncChannel()
    responses = client.connect(requests)
    await requests.send_from(
        [Message(body="Hello world 1"), Message(body="Hello world 2")], close=True
    )

    assert await to_list(responses) == expected_responses


@pytest.mark.asyncio
async def test_send_from_close_manually_immediately(client, expected_responses):
    requests = AsyncChannel()
    responses = client.connect(requests)
    await requests.send_from(
        [Message(body="Hello world 1"), Message(body="Hello world 2")], close=False
    )
    requests.close()

    assert await to_list(responses) == expected_responses


@pytest.mark.asyncio
async def test_send_individually_and_close_before_connect(client, expected_responses):
    requests = AsyncChannel()
    await requests.send(Message(body="Hello world 1"))
    await requests.send(Message(body="Hello world 2"))
    requests.close()
    responses = client.connect(requests)

    assert await to_list(responses) == expected_responses


@pytest.mark.asyncio
async def test_send_individually_and_close_after_connect(client, expected_responses):
    requests = AsyncChannel()
    await requests.send(Message(body="Hello world 1"))
    await requests.send(Message(body="Hello world 2"))
    responses = client.connect(requests)
    requests.close()

    assert await to_list(responses) == expected_responses



import pytest

import aristaproto
from tests.output_aristaproto.oneof_enum import (
    Move,
    Signal,
    Test,
)
from tests.util import get_test_case_json_data


def test_which_one_of_returns_enum_with_default_value():
    """
    returns first field when it is enum and set with default value
    """
    message = Test()
    message.from_json(
        get_test_case_json_data("oneof_enum", "oneof_enum-enum-0.json")[0].json
    )

    assert not hasattr(message, "move")
    assert object.__getattribute__(message, "move") == aristaproto.PLACEHOLDER
    assert message.signal == Signal.PASS
    assert aristaproto.which_one_of(message, "action") == ("signal", Signal.PASS)


def test_which_one_of_returns_enum_with_non_default_value():
    """
    returns first field when it is enum and set with non default value
    """
    message = Test()
    message.from_json(
        get_test_case_json_data("oneof_enum", "oneof_enum-enum-1.json")[0].json
    )
    assert not hasattr(message, "move")
    assert object.__getattribute__(message, "move") == aristaproto.PLACEHOLDER
    assert message.signal == Signal.RESIGN
    assert aristaproto.which_one_of(message, "action") == ("signal", Signal.RESIGN)


def test_which_one_of_returns_second_field_when_set():
    message = Test()
    message.from_json(get_test_case_json_data("oneof_enum")[0].json)
    assert message.move == Move(x=2, y=3)
    assert not hasattr(message, "signal")
    assert object.__getattribute__(message, "signal") == aristaproto.PLACEHOLDER
    assert aristaproto.which_one_of(message, "action") == ("move", Move(x=2, y=3))



import pytest

import aristaproto
from tests.output_aristaproto.oneof import (
    MixedDrink,
    Test,
)
from tests.output_aristaproto_pydantic.oneof import Test as TestPyd
from tests.util import get_test_case_json_data


def test_which_count():
    message = Test()
    message.from_json(get_test_case_json_data("oneof")[0].json)
    assert aristaproto.which_one_of(message, "foo") == ("pitied", 100)


def test_which_name():
    message = Test()
    message.from_json(get_test_case_json_data("oneof", "oneof_name.json")[0].json)
    assert aristaproto.which_one_of(message, "foo") == ("pitier", "Mr. T")


def test_which_count_pyd():
    message = TestPyd(pitier="Mr. T", just_a_regular_field=2, bar_name="a_bar")
    assert aristaproto.which_one_of(message, "foo") == ("pitier", "Mr. T")


def test_oneof_constructor_assign():
    message = Test(mixed_drink=MixedDrink(shots=42))
    field, value = aristaproto.which_one_of(message, "bar")
    assert field == "mixed_drink"
    assert value.shots == 42


# Issue #305:
@pytest.mark.xfail
def test_oneof_nested_assign():
    message = Test()
    message.mixed_drink.shots = 42
    field, value = aristaproto.which_one_of(message, "bar")
    assert field == "mixed_drink"
    assert value.shots == 42



import json
import sys
from copy import (
    copy,
    deepcopy,
)
from dataclasses import dataclass
from datetime import (
    datetime,
    timedelta,
)
from inspect import (
    Parameter,
    signature,
)
from typing import (
    Dict,
    List,
    Optional,
)
from unittest.mock import ANY

import pytest

import aristaproto


def test_has_field():
    @dataclass
    class Bar(aristaproto.Message):
        baz: int = aristaproto.int32_field(1)

    @dataclass
    class Foo(aristaproto.Message):
        bar: Bar = aristaproto.message_field(1)

    # Unset by default
    foo = Foo()
    assert aristaproto.serialized_on_wire(foo.bar) is False

    # Serialized after setting something
    foo.bar.baz = 1
    assert aristaproto.serialized_on_wire(foo.bar) is True

    # Still has it after setting the default value
    foo.bar.baz = 0
    assert aristaproto.serialized_on_wire(foo.bar) is True

    # Manual override (don't do this)
    foo.bar._serialized_on_wire = False
    assert aristaproto.serialized_on_wire(foo.bar) is False

    # Can manually set it but defaults to false
    foo.bar = Bar()
    assert aristaproto.serialized_on_wire(foo.bar) is False

    @dataclass
    class WithCollections(aristaproto.Message):
        test_list: List[str] = aristaproto.string_field(1)
        test_map: Dict[str, str] = aristaproto.map_field(
            2, aristaproto.TYPE_STRING, aristaproto.TYPE_STRING
        )

    # Is always set from parse, even if all collections are empty
    with_collections_empty = WithCollections().parse(bytes(WithCollections()))
    assert aristaproto.serialized_on_wire(with_collections_empty) == True
    with_collections_list = WithCollections().parse(
        bytes(WithCollections(test_list=["a", "b", "c"]))
    )
    assert aristaproto.serialized_on_wire(with_collections_list) == True
    with_collections_map = WithCollections().parse(
        bytes(WithCollections(test_map={"a": "b", "c": "d"}))
    )
    assert aristaproto.serialized_on_wire(with_collections_map) == True


def test_class_init():
    @dataclass
    class Bar(aristaproto.Message):
        name: str = aristaproto.string_field(1)

    @dataclass
    class Foo(aristaproto.Message):
        name: str = aristaproto.string_field(1)
        child: Bar = aristaproto.message_field(2)

    foo = Foo(name="foo", child=Bar(name="bar"))

    assert foo.to_dict() == {"name": "foo", "child": {"name": "bar"}}
    assert foo.to_pydict() == {"name": "foo", "child": {"name": "bar"}}


def test_enum_as_int_json():
    class TestEnum(aristaproto.Enum):
        ZERO = 0
        ONE = 1

    @dataclass
    class Foo(aristaproto.Message):
        bar: TestEnum = aristaproto.enum_field(1)

    # JSON strings are supported, but ints should still be supported too.
    foo = Foo().from_dict({"bar": 1})
    assert foo.bar == TestEnum.ONE

    # Plain-ol'-ints should serialize properly too.
    foo.bar = 1
    assert foo.to_dict() == {"bar": "ONE"}

    # Similar expectations for pydict
    foo = Foo().from_pydict({"bar": 1})
    assert foo.bar == TestEnum.ONE
    assert foo.to_pydict() == {"bar": TestEnum.ONE}


def test_unknown_fields():
    @dataclass
    class Newer(aristaproto.Message):
        foo: bool = aristaproto.bool_field(1)
        bar: int = aristaproto.int32_field(2)
        baz: str = aristaproto.string_field(3)

    @dataclass
    class Older(aristaproto.Message):
        foo: bool = aristaproto.bool_field(1)

    newer = Newer(foo=True, bar=1, baz="Hello")
    serialized_newer = bytes(newer)

    # Unknown fields in `Newer` should round trip with `Older`
    round_trip = bytes(Older().parse(serialized_newer))
    assert serialized_newer == round_trip

    new_again = Newer().parse(round_trip)
    assert newer == new_again


def test_oneof_support():
    @dataclass
    class Sub(aristaproto.Message):
        val: int = aristaproto.int32_field(1)

    @dataclass
    class Foo(aristaproto.Message):
        bar: int = aristaproto.int32_field(1, group="group1")
        baz: str = aristaproto.string_field(2, group="group1")
        sub: Sub = aristaproto.message_field(3, group="group2")
        abc: str = aristaproto.string_field(4, group="group2")

    foo = Foo()

    assert aristaproto.which_one_of(foo, "group1")[0] == ""

    foo.bar = 1
    foo.baz = "test"

    # Other oneof fields should now be unset
    assert not hasattr(foo, "bar")
    assert object.__getattribute__(foo, "bar") == aristaproto.PLACEHOLDER
    assert aristaproto.which_one_of(foo, "group1")[0] == "baz"

    foo.sub = Sub(val=1)
    assert aristaproto.serialized_on_wire(foo.sub)

    foo.abc = "test"

    # Group 1 shouldn't be touched, group 2 should have reset
    assert not hasattr(foo, "sub")
    assert object.__getattribute__(foo, "sub") == aristaproto.PLACEHOLDER
    assert aristaproto.which_one_of(foo, "group2")[0] == "abc"

    # Zero value should always serialize for one-of
    foo = Foo(bar=0)
    assert aristaproto.which_one_of(foo, "group1")[0] == "bar"
    assert bytes(foo) == b"\x08\x00"

    # Round trip should also work
    foo2 = Foo().parse(bytes(foo))
    assert aristaproto.which_one_of(foo2, "group1")[0] == "bar"
    assert foo.bar == 0
    assert aristaproto.which_one_of(foo2, "group2")[0] == ""


@pytest.mark.skipif(
    sys.version_info < (3, 10),
    reason="pattern matching is only supported in python3.10+",
)
def test_oneof_pattern_matching():
    from .oneof_pattern_matching import test_oneof_pattern_matching

    test_oneof_pattern_matching()


def test_json_casing():
    @dataclass
    class CasingTest(aristaproto.Message):
        pascal_case: int = aristaproto.int32_field(1)
        camel_case: int = aristaproto.int32_field(2)
        snake_case: int = aristaproto.int32_field(3)
        kabob_case: int = aristaproto.int32_field(4)

    # Parsing should accept almost any input
    test = CasingTest().from_dict(
        {"PascalCase": 1, "camelCase": 2, "snake_case": 3, "kabob-case": 4}
    )

    assert test == CasingTest(1, 2, 3, 4)

    # Serializing should be strict.
    assert json.loads(test.to_json()) == {
        "pascalCase": 1,
        "camelCase": 2,
        "snakeCase": 3,
        "kabobCase": 4,
    }

    assert json.loads(test.to_json(casing=aristaproto.Casing.SNAKE)) == {
        "pascal_case": 1,
        "camel_case": 2,
        "snake_case": 3,
        "kabob_case": 4,
    }


def test_dict_casing():
    @dataclass
    class CasingTest(aristaproto.Message):
        pascal_case: int = aristaproto.int32_field(1)
        camel_case: int = aristaproto.int32_field(2)
        snake_case: int = aristaproto.int32_field(3)
        kabob_case: int = aristaproto.int32_field(4)

    # Parsing should accept almost any input
    test = CasingTest().from_dict(
        {"PascalCase": 1, "camelCase": 2, "snake_case": 3, "kabob-case": 4}
    )

    assert test == CasingTest(1, 2, 3, 4)

    # Serializing should be strict.
    assert test.to_dict() == {
        "pascalCase": 1,
        "camelCase": 2,
        "snakeCase": 3,
        "kabobCase": 4,
    }
    assert test.to_pydict() == {
        "pascalCase": 1,
        "camelCase": 2,
        "snakeCase": 3,
        "kabobCase": 4,
    }

    assert test.to_dict(casing=aristaproto.Casing.SNAKE) == {
        "pascal_case": 1,
        "camel_case": 2,
        "snake_case": 3,
        "kabob_case": 4,
    }
    assert test.to_pydict(casing=aristaproto.Casing.SNAKE) == {
        "pascal_case": 1,
        "camel_case": 2,
        "snake_case": 3,
        "kabob_case": 4,
    }


def test_optional_flag():
    @dataclass
    class Request(aristaproto.Message):
        flag: Optional[bool] = aristaproto.message_field(1, wraps=aristaproto.TYPE_BOOL)

    # Serialization of not passed vs. set vs. zero-value.
    assert bytes(Request()) == b""
    assert bytes(Request(flag=True)) == b"\n\x02\x08\x01"
    assert bytes(Request(flag=False)) == b"\n\x00"

    # Differentiate between not passed and the zero-value.
    assert Request().parse(b"").flag is None
    assert Request().parse(b"\n\x00").flag is False


def test_optional_datetime_to_dict():
    @dataclass
    class Request(aristaproto.Message):
        date: Optional[datetime] = aristaproto.message_field(1, optional=True)

    # Check dict serialization
    assert Request().to_dict() == {}
    assert Request().to_dict(include_default_values=True) == {"date": None}
    assert Request(date=datetime(2020, 1, 1)).to_dict() == {
        "date": "2020-01-01T00:00:00Z"
    }
    assert Request(date=datetime(2020, 1, 1)).to_dict(include_default_values=True) == {
        "date": "2020-01-01T00:00:00Z"
    }

    # Check pydict serialization
    assert Request().to_pydict() == {}
    assert Request().to_pydict(include_default_values=True) == {"date": None}
    assert Request(date=datetime(2020, 1, 1)).to_pydict() == {
        "date": datetime(2020, 1, 1)
    }
    assert Request(date=datetime(2020, 1, 1)).to_pydict(
        include_default_values=True
    ) == {"date": datetime(2020, 1, 1)}


def test_to_json_default_values():
    @dataclass
    class TestMessage(aristaproto.Message):
        some_int: int = aristaproto.int32_field(1)
        some_double: float = aristaproto.double_field(2)
        some_str: str = aristaproto.string_field(3)
        some_bool: bool = aristaproto.bool_field(4)

    # Empty dict
    test = TestMessage().from_dict({})

    assert json.loads(test.to_json(include_default_values=True)) == {
        "someInt": 0,
        "someDouble": 0.0,
        "someStr": "",
        "someBool": False,
    }

    # All default values
    test = TestMessage().from_dict(
        {"someInt": 0, "someDouble": 0.0, "someStr": "", "someBool": False}
    )

    assert json.loads(test.to_json(include_default_values=True)) == {
        "someInt": 0,
        "someDouble": 0.0,
        "someStr": "",
        "someBool": False,
    }


def test_to_dict_default_values():
    @dataclass
    class TestMessage(aristaproto.Message):
        some_int: int = aristaproto.int32_field(1)
        some_double: float = aristaproto.double_field(2)
        some_str: str = aristaproto.string_field(3)
        some_bool: bool = aristaproto.bool_field(4)

    # Empty dict
    test = TestMessage().from_dict({})

    assert test.to_dict(include_default_values=True) == {
        "someInt": 0,
        "someDouble": 0.0,
        "someStr": "",
        "someBool": False,
    }

    test = TestMessage().from_pydict({})

    assert test.to_pydict(include_default_values=True) == {
        "someInt": 0,
        "someDouble": 0.0,
        "someStr": "",
        "someBool": False,
    }

    # All default values
    test = TestMessage().from_dict(
        {"someInt": 0, "someDouble": 0.0, "someStr": "", "someBool": False}
    )

    assert test.to_dict(include_default_values=True) == {
        "someInt": 0,
        "someDouble": 0.0,
        "someStr": "",
        "someBool": False,
    }

    test = TestMessage().from_pydict(
        {"someInt": 0, "someDouble": 0.0, "someStr": "", "someBool": False}
    )

    assert test.to_pydict(include_default_values=True) == {
        "someInt": 0,
        "someDouble": 0.0,
        "someStr": "",
        "someBool": False,
    }

    # Some default and some other values
    @dataclass
    class TestMessage2(aristaproto.Message):
        some_int: int = aristaproto.int32_field(1)
        some_double: float = aristaproto.double_field(2)
        some_str: str = aristaproto.string_field(3)
        some_bool: bool = aristaproto.bool_field(4)
        some_default_int: int = aristaproto.int32_field(5)
        some_default_double: float = aristaproto.double_field(6)
        some_default_str: str = aristaproto.string_field(7)
        some_default_bool: bool = aristaproto.bool_field(8)

    test = TestMessage2().from_dict(
        {
            "someInt": 2,
            "someDouble": 1.2,
            "someStr": "hello",
            "someBool": True,
            "someDefaultInt": 0,
            "someDefaultDouble": 0.0,
            "someDefaultStr": "",
            "someDefaultBool": False,
        }
    )

    assert test.to_dict(include_default_values=True) == {
        "someInt": 2,
        "someDouble": 1.2,
        "someStr": "hello",
        "someBool": True,
        "someDefaultInt": 0,
        "someDefaultDouble": 0.0,
        "someDefaultStr": "",
        "someDefaultBool": False,
    }

    test = TestMessage2().from_pydict(
        {
            "someInt": 2,
            "someDouble": 1.2,
            "someStr": "hello",
            "someBool": True,
            "someDefaultInt": 0,
            "someDefaultDouble": 0.0,
            "someDefaultStr": "",
            "someDefaultBool": False,
        }
    )

    assert test.to_pydict(include_default_values=True) == {
        "someInt": 2,
        "someDouble": 1.2,
        "someStr": "hello",
        "someBool": True,
        "someDefaultInt": 0,
        "someDefaultDouble": 0.0,
        "someDefaultStr": "",
        "someDefaultBool": False,
    }

    # Nested messages
    @dataclass
    class TestChildMessage(aristaproto.Message):
        some_other_int: int = aristaproto.int32_field(1)

    @dataclass
    class TestParentMessage(aristaproto.Message):
        some_int: int = aristaproto.int32_field(1)
        some_double: float = aristaproto.double_field(2)
        some_message: TestChildMessage = aristaproto.message_field(3)

    test = TestParentMessage().from_dict({"someInt": 0, "someDouble": 1.2})

    assert test.to_dict(include_default_values=True) == {
        "someInt": 0,
        "someDouble": 1.2,
        "someMessage": {"someOtherInt": 0},
    }

    test = TestParentMessage().from_pydict({"someInt": 0, "someDouble": 1.2})

    assert test.to_pydict(include_default_values=True) == {
        "someInt": 0,
        "someDouble": 1.2,
        "someMessage": {"someOtherInt": 0},
    }


def test_to_dict_datetime_values():
    @dataclass
    class TestDatetimeMessage(aristaproto.Message):
        bar: datetime = aristaproto.message_field(1)
        baz: timedelta = aristaproto.message_field(2)

    test = TestDatetimeMessage().from_dict(
        {"bar": "2020-01-01T00:00:00Z", "baz": "86400.000s"}
    )

    assert test.to_dict() == {"bar": "2020-01-01T00:00:00Z", "baz": "86400.000s"}

    test = TestDatetimeMessage().from_pydict(
        {"bar": datetime(year=2020, month=1, day=1), "baz": timedelta(days=1)}
    )

    assert test.to_pydict() == {
        "bar": datetime(year=2020, month=1, day=1),
        "baz": timedelta(days=1),
    }


def test_oneof_default_value_set_causes_writes_wire():
    @dataclass
    class Empty(aristaproto.Message):
        pass

    @dataclass
    class Foo(aristaproto.Message):
        bar: int = aristaproto.int32_field(1, group="group1")
        baz: str = aristaproto.string_field(2, group="group1")
        qux: Empty = aristaproto.message_field(3, group="group1")

    def _round_trip_serialization(foo: Foo) -> Foo:
        return Foo().parse(bytes(foo))

    foo1 = Foo(bar=0)
    foo2 = Foo(baz="")
    foo3 = Foo(qux=Empty())
    foo4 = Foo()

    assert bytes(foo1) == b"\x08\x00"
    assert (
        aristaproto.which_one_of(foo1, "group1")
        == aristaproto.which_one_of(_round_trip_serialization(foo1), "group1")
        == ("bar", 0)
    )

    assert bytes(foo2) == b"\x12\x00"  # Baz is just an empty string
    assert (
        aristaproto.which_one_of(foo2, "group1")
        == aristaproto.which_one_of(_round_trip_serialization(foo2), "group1")
        == ("baz", "")
    )

    assert bytes(foo3) == b"\x1a\x00"
    assert (
        aristaproto.which_one_of(foo3, "group1")
        == aristaproto.which_one_of(_round_trip_serialization(foo3), "group1")
        == ("qux", Empty())
    )

    assert bytes(foo4) == b""
    assert (
        aristaproto.which_one_of(foo4, "group1")
        == aristaproto.which_one_of(_round_trip_serialization(foo4), "group1")
        == ("", None)
    )


def test_message_repr():
    from tests.output_aristaproto.recursivemessage import Test

    assert repr(Test(name="Loki")) == "Test(name='Loki')"
    assert repr(Test(child=Test(), name="Loki")) == "Test(name='Loki', child=Test())"


def test_bool():
    """Messages should evaluate similarly to a collection
    >>> test = []
    >>> bool(test)
    ... False
    >>> test.append(1)
    >>> bool(test)
    ... True
    >>> del test[0]
    >>> bool(test)
    ... False
    """

    @dataclass
    class Falsy(aristaproto.Message):
        pass

    @dataclass
    class Truthy(aristaproto.Message):
        bar: int = aristaproto.int32_field(1)

    assert not Falsy()
    t = Truthy()
    assert not t
    t.bar = 1
    assert t
    t.bar = 0
    assert not t


# valid ISO datetimes according to https://www.myintervals.com/blog/2009/05/20/iso-8601-date-validation-that-doesnt-suck/
iso_candidates = """2009-12-12T12:34
2009
2009-05-19
2009-05-19
20090519
2009123
2009-05
2009-123
2009-222
2009-001
2009-W01-1
2009-W51-1
2009-W33
2009W511
2009-05-19
2009-05-19 00:00
2009-05-19 14
2009-05-19 14:31
2009-05-19 14:39:22
2009-05-19T14:39Z
2009-W21-2
2009-W21-2T01:22
2009-139
2009-05-19 14:39:22-06:00
2009-05-19 14:39:22+0600
2009-05-19 14:39:22-01
20090621T0545Z
2007-04-06T00:00
2007-04-05T24:00
2010-02-18T16:23:48.5
2010-02-18T16:23:48,444
2010-02-18T16:23:48,3-06:00
2010-02-18T16:23:00.4
2010-02-18T16:23:00,25
2010-02-18T16:23:00.33+0600
2010-02-18T16:00:00.23334444
2010-02-18T16:00:00,2283
2009-05-19 143922
2009-05-19 1439""".split(
    "\n"
)


def test_iso_datetime():
    @dataclass
    class Envelope(aristaproto.Message):
        ts: datetime = aristaproto.message_field(1)

    msg = Envelope()

    for _, candidate in enumerate(iso_candidates):
        msg.from_dict({"ts": candidate})
        assert isinstance(msg.ts, datetime)


def test_iso_datetime_list():
    @dataclass
    class Envelope(aristaproto.Message):
        timestamps: List[datetime] = aristaproto.message_field(1)

    msg = Envelope()

    msg.from_dict({"timestamps": iso_candidates})
    assert all([isinstance(item, datetime) for item in msg.timestamps])


def test_service_argument__expected_parameter():
    from tests.output_aristaproto.service import TestStub

    sig = signature(TestStub.do_thing)
    do_thing_request_parameter = sig.parameters["do_thing_request"]
    assert do_thing_request_parameter.default is Parameter.empty
    assert do_thing_request_parameter.annotation == "DoThingRequest"


def test_is_set():
    @dataclass
    class Spam(aristaproto.Message):
        foo: bool = aristaproto.bool_field(1)
        bar: Optional[int] = aristaproto.int32_field(2, optional=True)

    assert not Spam().is_set("foo")
    assert not Spam().is_set("bar")
    assert Spam(foo=True).is_set("foo")
    assert Spam(foo=True, bar=0).is_set("bar")


def test_equality_comparison():
    from tests.output_aristaproto.bool import Test as TestMessage

    msg = TestMessage(value=True)

    assert msg == msg
    assert msg == ANY
    assert msg == TestMessage(value=True)
    assert msg != 1
    assert msg != TestMessage(value=False)



#!/usr/bin/env python
import asyncio
import os
import platform
import shutil
import sys
from pathlib import Path
from typing import Set

from tests.util import (
    get_directories,
    inputs_path,
    output_path_aristaproto,
    output_path_aristaproto_pydantic,
    output_path_reference,
    protoc,
)


# Force pure-python implementation instead of C++, otherwise imports
# break things because we can't properly reset the symbol database.
os.environ["PROTOCOL_BUFFERS_PYTHON_IMPLEMENTATION"] = "python"


def clear_directory(dir_path: Path):
    for file_or_directory in dir_path.glob("*"):
        if file_or_directory.is_dir():
            shutil.rmtree(file_or_directory)
        else:
            file_or_directory.unlink()


async def generate(whitelist: Set[str], verbose: bool):
    test_case_names = set(get_directories(inputs_path)) - {"__pycache__"}

    path_whitelist = set()
    name_whitelist = set()
    for item in whitelist:
        if item in test_case_names:
            name_whitelist.add(item)
            continue
        path_whitelist.add(item)

    generation_tasks = []
    for test_case_name in sorted(test_case_names):
        test_case_input_path = inputs_path.joinpath(test_case_name).resolve()
        if (
            whitelist
            and str(test_case_input_path) not in path_whitelist
            and test_case_name not in name_whitelist
        ):
            continue
        generation_tasks.append(
            generate_test_case_output(test_case_input_path, test_case_name, verbose)
        )

    failed_test_cases = []
    # Wait for all subprocs and match any failures to names to report
    for test_case_name, result in zip(
        sorted(test_case_names), await asyncio.gather(*generation_tasks)
    ):
        if result != 0:
            failed_test_cases.append(test_case_name)

    if len(failed_test_cases) > 0:
        sys.stderr.write(
            "\n\033[31;1;4mFailed to generate the following test cases:\033[0m\n"
        )
        for failed_test_case in failed_test_cases:
            sys.stderr.write(f"- {failed_test_case}\n")

        sys.exit(1)


async def generate_test_case_output(
    test_case_input_path: Path, test_case_name: str, verbose: bool
) -> int:
    """
    Returns the max of the subprocess return values
    """

    test_case_output_path_reference = output_path_reference.joinpath(test_case_name)
    test_case_output_path_aristaproto = output_path_aristaproto
    test_case_output_path_aristaproto_pyd = output_path_aristaproto_pydantic

    os.makedirs(test_case_output_path_reference, exist_ok=True)
    os.makedirs(test_case_output_path_aristaproto, exist_ok=True)
    os.makedirs(test_case_output_path_aristaproto_pyd, exist_ok=True)

    clear_directory(test_case_output_path_reference)
    clear_directory(test_case_output_path_aristaproto)

    (
        (ref_out, ref_err, ref_code),
        (plg_out, plg_err, plg_code),
        (plg_out_pyd, plg_err_pyd, plg_code_pyd),
    ) = await asyncio.gather(
        protoc(test_case_input_path, test_case_output_path_reference, True),
        protoc(test_case_input_path, test_case_output_path_aristaproto, False),
        protoc(
            test_case_input_path, test_case_output_path_aristaproto_pyd, False, True
        ),
    )

    if ref_code == 0:
        print(f"\033[31;1;4mGenerated reference output for {test_case_name!r}\033[0m")
    else:
        print(
            f"\033[31;1;4mFailed to generate reference output for {test_case_name!r}\033[0m"
        )

    if verbose:
        if ref_out:
            print("Reference stdout:")
            sys.stdout.buffer.write(ref_out)
            sys.stdout.buffer.flush()

        if ref_err:
            print("Reference stderr:")
            sys.stderr.buffer.write(ref_err)
            sys.stderr.buffer.flush()

    if plg_code == 0:
        print(f"\033[31;1;4mGenerated plugin output for {test_case_name!r}\033[0m")
    else:
        print(
            f"\033[31;1;4mFailed to generate plugin output for {test_case_name!r}\033[0m"
        )

    if verbose:
        if plg_out:
            print("Plugin stdout:")
            sys.stdout.buffer.write(plg_out)
            sys.stdout.buffer.flush()

        if plg_err:
            print("Plugin stderr:")
            sys.stderr.buffer.write(plg_err)
            sys.stderr.buffer.flush()

    if plg_code_pyd == 0:
        print(
            f"\033[31;1;4mGenerated plugin (pydantic compatible) output for {test_case_name!r}\033[0m"
        )
    else:
        print(
            f"\033[31;1;4mFailed to generate plugin (pydantic compatible) output for {test_case_name!r}\033[0m"
        )

    if verbose:
        if plg_out_pyd:
            print("Plugin stdout:")
            sys.stdout.buffer.write(plg_out_pyd)
            sys.stdout.buffer.flush()

        if plg_err_pyd:
            print("Plugin stderr:")
            sys.stderr.buffer.write(plg_err_pyd)
            sys.stderr.buffer.flush()

    return max(ref_code, plg_code, plg_code_pyd)


HELP = "\n".join(
    (
        "Usage: python generate.py [-h] [-v] [DIRECTORIES or NAMES]",
        "Generate python classes for standard tests.",
        "",
        "DIRECTORIES    One or more relative or absolute directories of test-cases to generate classes for.",
        "               python generate.py inputs/bool inputs/double inputs/enum",
        "",
        "NAMES          One or more test-case names to generate classes for.",
        "               python generate.py bool double enums",
    )
)


def main():
    if set(sys.argv).intersection({"-h", "--help"}):
        print(HELP)
        return
    if sys.argv[1:2] == ["-v"]:
        verbose = True
        whitelist = set(sys.argv[2:])
    else:
        verbose = False
        whitelist = set(sys.argv[1:])

    if platform.system() == "Windows":
        asyncio.set_event_loop_policy(asyncio.WindowsProactorEventLoopPolicy())

    asyncio.run(generate(whitelist, verbose))


if __name__ == "__main__":
    main()



import asyncio
import sys
import uuid

import grpclib
import grpclib.client
import grpclib.metadata
import grpclib.server
import pytest
from grpclib.testing import ChannelFor

from aristaproto.grpc.util.async_channel import AsyncChannel
from tests.output_aristaproto.service import (
    DoThingRequest,
    DoThingResponse,
    GetThingRequest,
    TestStub as ThingServiceClient,
)

from .thing_service import ThingService


async def _test_client(client: ThingServiceClient, name="clean room", **kwargs):
    response = await client.do_thing(DoThingRequest(name=name), **kwargs)
    assert response.names == [name]


def _assert_request_meta_received(deadline, metadata):
    def server_side_test(stream):
        assert stream.deadline._timestamp == pytest.approx(
            deadline._timestamp, 1
        ), "The provided deadline should be received serverside"
        assert (
            stream.metadata["authorization"] == metadata["authorization"]
        ), "The provided authorization metadata should be received serverside"

    return server_side_test


@pytest.fixture
def handler_trailer_only_unauthenticated():
    async def handler(stream: grpclib.server.Stream):
        await stream.recv_message()
        await stream.send_initial_metadata()
        await stream.send_trailing_metadata(status=grpclib.Status.UNAUTHENTICATED)

    return handler


@pytest.mark.asyncio
async def test_simple_service_call():
    async with ChannelFor([ThingService()]) as channel:
        await _test_client(ThingServiceClient(channel))


@pytest.mark.asyncio
async def test_trailer_only_error_unary_unary(
    mocker, handler_trailer_only_unauthenticated
):
    service = ThingService()
    mocker.patch.object(
        service,
        "do_thing",
        side_effect=handler_trailer_only_unauthenticated,
        autospec=True,
    )
    async with ChannelFor([service]) as channel:
        with pytest.raises(grpclib.exceptions.GRPCError) as e:
            await ThingServiceClient(channel).do_thing(DoThingRequest(name="something"))
        assert e.value.status == grpclib.Status.UNAUTHENTICATED


@pytest.mark.asyncio
async def test_trailer_only_error_stream_unary(
    mocker, handler_trailer_only_unauthenticated
):
    service = ThingService()
    mocker.patch.object(
        service,
        "do_many_things",
        side_effect=handler_trailer_only_unauthenticated,
        autospec=True,
    )
    async with ChannelFor([service]) as channel:
        with pytest.raises(grpclib.exceptions.GRPCError) as e:
            await ThingServiceClient(channel).do_many_things(
                do_thing_request_iterator=[DoThingRequest(name="something")]
            )
            await _test_client(ThingServiceClient(channel))
        assert e.value.status == grpclib.Status.UNAUTHENTICATED


@pytest.mark.asyncio
@pytest.mark.skipif(
    sys.version_info < (3, 8), reason="async mock spy does works for python3.8+"
)
async def test_service_call_mutable_defaults(mocker):
    async with ChannelFor([ThingService()]) as channel:
        client = ThingServiceClient(channel)
        spy = mocker.spy(client, "_unary_unary")
        await _test_client(client)
        comments = spy.call_args_list[-1].args[1].comments
        await _test_client(client)
        assert spy.call_args_list[-1].args[1].comments is not comments


@pytest.mark.asyncio
async def test_service_call_with_upfront_request_params():
    # Setting deadline
    deadline = grpclib.metadata.Deadline.from_timeout(22)
    metadata = {"authorization": "12345"}
    async with ChannelFor(
        [ThingService(test_hook=_assert_request_meta_received(deadline, metadata))]
    ) as channel:
        await _test_client(
            ThingServiceClient(channel, deadline=deadline, metadata=metadata)
        )

    # Setting timeout
    timeout = 99
    deadline = grpclib.metadata.Deadline.from_timeout(timeout)
    metadata = {"authorization": "12345"}
    async with ChannelFor(
        [ThingService(test_hook=_assert_request_meta_received(deadline, metadata))]
    ) as channel:
        await _test_client(
            ThingServiceClient(channel, timeout=timeout, metadata=metadata)
        )


@pytest.mark.asyncio
async def test_service_call_lower_level_with_overrides():
    THING_TO_DO = "get milk"

    # Setting deadline
    deadline = grpclib.metadata.Deadline.from_timeout(22)
    metadata = {"authorization": "12345"}
    kwarg_deadline = grpclib.metadata.Deadline.from_timeout(28)
    kwarg_metadata = {"authorization": "12345"}
    async with ChannelFor(
        [ThingService(test_hook=_assert_request_meta_received(deadline, metadata))]
    ) as channel:
        client = ThingServiceClient(channel, deadline=deadline, metadata=metadata)
        response = await client._unary_unary(
            "/service.Test/DoThing",
            DoThingRequest(THING_TO_DO),
            DoThingResponse,
            deadline=kwarg_deadline,
            metadata=kwarg_metadata,
        )
        assert response.names == [THING_TO_DO]

    # Setting timeout
    timeout = 99
    deadline = grpclib.metadata.Deadline.from_timeout(timeout)
    metadata = {"authorization": "12345"}
    kwarg_timeout = 9000
    kwarg_deadline = grpclib.metadata.Deadline.from_timeout(kwarg_timeout)
    kwarg_metadata = {"authorization": "09876"}
    async with ChannelFor(
        [
            ThingService(
                test_hook=_assert_request_meta_received(kwarg_deadline, kwarg_metadata),
            )
        ]
    ) as channel:
        client = ThingServiceClient(channel, deadline=deadline, metadata=metadata)
        response = await client._unary_unary(
            "/service.Test/DoThing",
            DoThingRequest(THING_TO_DO),
            DoThingResponse,
            timeout=kwarg_timeout,
            metadata=kwarg_metadata,
        )
        assert response.names == [THING_TO_DO]


@pytest.mark.asyncio
@pytest.mark.parametrize(
    ("overrides_gen",),
    [
        (lambda: dict(timeout=10),),
        (lambda: dict(deadline=grpclib.metadata.Deadline.from_timeout(10)),),
        (lambda: dict(metadata={"authorization": str(uuid.uuid4())}),),
        (lambda: dict(timeout=20, metadata={"authorization": str(uuid.uuid4())}),),
    ],
)
async def test_service_call_high_level_with_overrides(mocker, overrides_gen):
    overrides = overrides_gen()
    request_spy = mocker.spy(grpclib.client.Channel, "request")
    name = str(uuid.uuid4())
    defaults = dict(
        timeout=99,
        deadline=grpclib.metadata.Deadline.from_timeout(99),
        metadata={"authorization": name},
    )

    async with ChannelFor(
        [
            ThingService(
                test_hook=_assert_request_meta_received(
                    deadline=grpclib.metadata.Deadline.from_timeout(
                        overrides.get("timeout", 99)
                    ),
                    metadata=overrides.get("metadata", defaults.get("metadata")),
                )
            )
        ]
    ) as channel:
        client = ThingServiceClient(channel, **defaults)
        await _test_client(client, name=name, **overrides)
        assert request_spy.call_count == 1

        # for python <3.8 request_spy.call_args.kwargs do not work
        _, request_spy_call_kwargs = request_spy.call_args_list[0]

        # ensure all overrides were successful
        for key, value in overrides.items():
            assert key in request_spy_call_kwargs
            assert request_spy_call_kwargs[key] == value

        # ensure default values were retained
        for key in set(defaults.keys()) - set(overrides.keys()):
            assert key in request_spy_call_kwargs
            assert request_spy_call_kwargs[key] == defaults[key]


@pytest.mark.asyncio
async def test_async_gen_for_unary_stream_request():
    thing_name = "my milkshakes"

    async with ChannelFor([ThingService()]) as channel:
        client = ThingServiceClient(channel)
        expected_versions = [5, 4, 3, 2, 1]
        async for response in client.get_thing_versions(
            GetThingRequest(name=thing_name)
        ):
            assert response.name == thing_name
            assert response.version == expected_versions.pop()


@pytest.mark.asyncio
async def test_async_gen_for_stream_stream_request():
    some_things = ["cake", "cricket", "coral reef"]
    more_things = ["ball", "that", "56kmodem", "liberal humanism", "cheesesticks"]
    expected_things = (*some_things, *more_things)

    async with ChannelFor([ThingService()]) as channel:
        client = ThingServiceClient(channel)
        # Use an AsyncChannel to decouple sending and recieving, it'll send some_things
        # immediately and we'll use it to send more_things later, after recieving some
        # results
        request_chan = AsyncChannel()
        send_initial_requests = asyncio.ensure_future(
            request_chan.send_from(GetThingRequest(name) for name in some_things)
        )
        response_index = 0
        async for response in client.get_different_things(request_chan):
            assert response.name == expected_things[response_index]
            assert response.version == response_index + 1
            response_index += 1
            if more_things:
                # Send some more requests as we receive responses to be sure coordination of
                # send/receive events doesn't matter
                await request_chan.send(GetThingRequest(more_things.pop(0)))
            elif not send_initial_requests.done():
                # Make sure the sending task it completed
                await send_initial_requests
            else:
                # No more things to send make sure channel is closed
                request_chan.close()
        assert response_index == len(
            expected_things
        ), "Didn't receive all expected responses"


@pytest.mark.asyncio
async def test_stream_unary_with_empty_iterable():
    things = []  # empty

    async with ChannelFor([ThingService()]) as channel:
        client = ThingServiceClient(channel)
        requests = [DoThingRequest(name) for name in things]
        response = await client.do_many_things(requests)
        assert len(response.names) == 0


@pytest.mark.asyncio
async def test_stream_stream_with_empty_iterable():
    things = []  # empty

    async with ChannelFor([ThingService()]) as channel:
        client = ThingServiceClient(channel)
        requests = [GetThingRequest(name) for name in things]
        responses = [
            response async for response in client.get_different_things(requests)
        ]
        assert len(responses) == 0
